/*
   Adapted from code provided by Darryl Hurley, Implestrat.
*/

DECLARE
   l_object   json_object_t;
BEGIN
   -- Elements can be added to an object...
   -- ...conditionally...
   -- ...overloaded...
   -- ...and removed...
   -- ...or renamed...
   l_object := json_object_t.parse ('{"Name":"Darryl"}');
   l_object.put ('Nickname', 'The King');
   DBMS_OUTPUT.put_line (l_object.to_string);
   -- PUT is an up-sert!
   l_object.put ('Nickname', 'Not The King');
   DBMS_OUTPUT.put_line (l_object.to_string);
   l_object.put ('Age', 11);
   DBMS_OUTPUT.put_line (l_object.to_string);

   IF NOT l_object.has ('Age')
   THEN
      l_object.put ('Age', 22);
   END IF;

   l_object.remove ('Age');
   DBMS_OUTPUT.put_line (l_object.to_string);
   l_object.rename_key ('Nickname', 'RealityIs');
   DBMS_OUTPUT.put_line (l_object.to_string);
END;
/

DECLARE
   l_object   json_object_t;
BEGIN
   -- remove is forgiving
   l_object := json_object_t.parse ('{"Name":"Darryl"}');
   l_object.remove ('NotThere');
   DBMS_OUTPUT.put_line (l_object.to_string);
END;
/

DECLARE
   l_object   json_object_t;
BEGIN
   -- rename is not forgiving
   l_object := json_object_t.parse ('{"Name":"Darryl"}');
   l_object.rename_key ('NotThere', 'DudeItsNotThere');
   DBMS_OUTPUT.put_line (l_object.to_string);
END;
/

DECLARE
   l_object            json_object_t;
   e_singing_off_key   EXCEPTION;
   PRAGMA EXCEPTION_INIT (e_singing_off_key, -40571);
BEGIN
   -- plsql to the rescue!
   l_object := json_object_t.parse ('{"Name":"Darryl"}');

   BEGIN
      l_object.rename_key ('NotThere', 'DudeItsNotThere');
   EXCEPTION
      WHEN e_singing_off_key
      THEN
         NULL;
      WHEN OTHERS
      THEN
         RAISE;
   END;

   DBMS_OUTPUT.put_line (l_object.to_string);
END;
/