ALTER SESSION SET plscope_settings='identifiers:all, statements:all'
/


/* Find all possible dynamic SQL injection points.
   Note: DBMS_SQL must be recompiled with PL/Scope turned on
   to get any results from the second half of the union all. */

SELECT st.owner,
       st.object_name,
       st.line,
       s.text
  FROM all_statements st, all_source s
 WHERE     st.TYPE IN ('EXECUTE IMMEDIATE', 'OPEN')
       AND st.owner = s.owner
       AND st.object_name = s.name
       AND st.line = s.line
UNION ALL
SELECT idnt.owner,
       idnt.object_name,
       idnt.line,
       src.text
  FROM all_identifiers idnt, all_source src
 WHERE     idnt.owner <> 'SYS'
       AND idnt.signature IN (SELECT a.signature
                                FROM all_identifiers a
                               WHERE     a.usage = 'DECLARATION'
                                     AND a.owner = 'SYS'
                                     AND a.object_name = 'DBMS_SQL'
                                     AND a.object_type = 'PACKAGE')
       AND idnt.owner = src.owner
       AND idnt.name = src.name
       AND idnt.line = src.line
/

/* If DBMS_SQL identifiers have not been gathered, "dumb" text search */


SELECT st.owner,
       st.object_name,
       st.line,
       s.text
  FROM all_statements st, all_source s
 WHERE     st.TYPE IN ('EXECUTE IMMEDIATE', 'OPEN')
       AND st.owner = s.owner
       AND st.object_name = s.name
       AND st.line = s.line
UNION ALL
SELECT src.owner,
       src.name,
       src.line,
       src.text
  FROM all_source src
 WHERE text like '%DBMS_SQL%
/