cl scr
set serveroutput on
/* pre-18c */
declare
  arr dbms_sql.number_table;
begin

  for i in 1 .. 10 loop  
    arr ( i ) := i;  
  end loop;
  
  for i in 1 .. arr.count loop
    dbms_output.put_line ( 'Val = ' || arr ( i ) );    
  end loop;

end;
/

/* 18c qualified expressions (aka type constructors) */
declare
  arr dbms_sql.number_table;
begin

  arr := dbms_sql.number_table (
    1 => 1, 2 => 2, 3 => 3, 4 => 4 --etc.
  );
  
  for i in 1 .. arr.count loop
    dbms_output.put_line ( 'Val = ' || arr ( i ) );
  end loop;

end;
/

/* 20c - use iterators in constructor! */
declare
  arr dbms_sql.number_table;
begin
  arr := dbms_sql.number_table ( 
    for i in 1 .. 10 => i
  );
  for i in indices of arr loop
    dbms_output.put_line ( 'Val = ' || arr(i) );
  end loop;
end;
/


/* 20c - can use extended iterators */
declare
  arr dbms_sql.number_table;
begin
  arr := dbms_sql.number_table ( 
    for i in 0 .. 8 by 2, 16 .. 20 => i
  );
  for i in indices of arr loop
    dbms_output.put_line ( 'Index: ' || i || ' value = ' || arr(i) );
  end loop;
end;
/


/* Combine arrays :) */
declare
  arr1 dbms_sql.number_table;
  arr2 dbms_sql.number_table;
  arr12 dbms_sql.number_table;
begin
  arr1 := dbms_sql.number_table ( 
    for i in 1 .. 5 => i
  );
  arr2 := dbms_sql.number_table ( 
    for i in 11 .. 15 => i * 2
  );
  
  arr12 := dbms_sql.number_table ( 
    for v in indices of arr1, values of arr2 => v
  );
  
  for i in indices of arr12 loop
    dbms_output.put_line ( 'Index ' || i || ' = ' || arr12(i) );
  end loop;
end;
/


/* Specify separate index */
declare
  arr dbms_sql.number_table;
begin

  arr := dbms_sql.number_table ( 
    for i in 1 .. 5 index i * 2 => i
  );
  
  for i in indices of arr loop
    dbms_output.put_line ( 'Index: ' || i || ' = ' || arr(i) );
  end loop;
  
end;
/








declare
  arr12 dbms_sql.number_table;
  arr1 dbms_sql.number_table;
  arr2 dbms_sql.number_table;
begin
  arr1 := dbms_sql.number_table ( 
    for i in 1 .. 5 => i
  );
  arr2 := dbms_sql.number_table ( 
    for i in 10 .. 15 => i * 2
  );
  arr12 := dbms_sql.number_table ( 
    for i, v in pairs of arr1, pairs of arr2 index i => v
  );
  for i in indices of arr12 loop
    dbms_output.put_line ( 'Index ' || i || ' = ' || arr12(i) );
  end loop;
end;
/


/* Combine arrays of complex types */
declare
  type rec is record ( 
    a1 int, a2 varchar2(1)
  );
  type rec_arr is table of rec
    index by pls_integer;
  arr1 rec_arr;
  arr2 rec_arr;
  arr12 rec_arr;
begin
  arr1 := rec_arr ( 
    1 => rec ( 1, 'a' ), 2 => rec ( 2, 'b' )
  );
  
  arr2 := rec_arr ( 
    3 => rec ( 3, 'c' ), 4 => rec ( 4, 'd' )
  );
  
  arr12 := rec_arr (
    for inx, val in pairs of arr1, 
      pairs of arr2 
      index inx => val
  );
  
  for inx, val in pairs of arr12 loop
--    dbms_output.put_line ( inx || ', ' || arr12( inx ).a1 || '-' || arr12( inx ).a2 );
    dbms_output.put_line ( inx || ', ' || val.a1 || '-' || val.a2 );
  end loop;
end;
/


/* Index by character init */
declare
  type arr is table of varchar2(1) 
    index by varchar2(1);
  a arr;
begin
  a := arr ( 
    for i in 1 .. 26 index chr ( i + 64 ) => 
      chr ( i + 64 ) 
  );
  
  for v in indices of a loop
    dbms_output.put_line ( v || ' is ' || a(v) ); 
  end loop;
end;
/



declare
  cursor cur is 
    select level c1 from dual
    connect by level <= 10;
    
  type cur_arr is table of cur%rowtype 
    index by pls_integer;
    
  a cur_arr;
begin
  a := cur_arr (
    for rws in cur index rws.c1 => rws
  );
  
  for i in indices of a loop
    dbms_output.put_line ( a ( i ).c1 );
  end loop;
  
end;
/

drop table currency_exrates
  cascade constraints purge;
create table currency_exrates (
  from_currency_code varchar2(3),
  to_currency_code   varchar2(3),
  exrate             number,
  primary key ( 
    from_currency_code, to_currency_code 
  )
);

insert into currency_exrates 
  values ( 'GBP', 'EUR', 1.09 );
insert into currency_exrates 
  values ( 'GBP', 'USD', 1.27 );
insert into currency_exrates 
  values ( 'GBP', 'INR', 96.19 );
insert into currency_exrates 
  values ( 'GBP', 'CAD', 1.7 );
insert into currency_exrates 
  values ( 'GBP', 'CHF', 1.17 );
commit;


declare

  type ex_arr is table of number 
    index by varchar2(3);
  
  ex_rates ex_arr;
  curr     currency_exrates.to_currency_code%type;
  
begin
  
  for currs in (
    select to_currency_code, exrate 
    from   currency_exrates
    where  from_currency_code = 'GBP'
  ) loop
  
    ex_rates ( currs.to_currency_code ) := currs.exrate;
    
  end loop;
  
  curr := ex_rates.first;
  while curr is not null loop
    dbms_output.put_line ( 
      'Currency ' || curr || ' exrate ' || ex_rates ( curr ) 
    );
    curr := ex_rates.next ( curr );
  end loop;

end;
/


/* Load a index by VC collection from a cursor */
declare
  cursor cur is 
    select to_currency_code, exrate 
    from   currency_exrates
    where  from_currency_code = 'GBP';
  
  type ex_arr is table of currency_exrates.exrate%type
    index by currency_exrates.to_currency_code%type;
  ex_rates ex_arr;
begin

  ex_rates := ex_arr (
     for currs cur%rowtype in cur 
       index currs.to_currency_code 
       => currs.exrate
  );
  
  for curr, rate in pairs of ex_rates loop
    dbms_output.put_line ( 
      'Currency ' || curr || ' rate = ' || rate
    );   
  end loop;
  
end;
/
  



/***********************************************/
/***********************************************/
/***********************************************/















/***********************************************/

/* */
declare
  stmt varchar2(100) := 'select level c1, mod ( level, 3 ) c2 from dual connect by level <= 10';
  type rec is record (
    c1 integer, c2 integer
  );
  type rec_arr is table of rec index by pls_integer;
  a rec_arr;
begin

  a := rec_arr (
    for rws rec in ( 
      execute immediate stmt 
    ) index rws.c1 => rws
  );
  
  for i in indices of a loop
    dbms_output.put_line ( a ( i ).c2 );
  end loop;
  
end;
/

declare
  stmt varchar2(100) := 'select chr ( level + 64 ) c1, mod ( level, 3 ) c2 from dual connect by level <= 10';
  type rec is record (
    c1 varchar2(1), c2 integer
  );
  type rec_arr is table of rec index by varchar2(1);
  a rec_arr;
begin

  a := rec_arr (
    for rws rec in ( 
      execute immediate stmt 
    ) index rws.c1 => rws
  );
  
  for i in indices of a loop
    dbms_output.put_line ( i );
  end loop;
  
end;
/



/* Dynamic SQL too */
declare
  stmt varchar2(100) := 'select level c1, mod ( level, 3 ) c2 from dual connect by level <= 10';
  type rec is record (
    c1 integer, c2 integer
  );
begin
  for rws rec in values of ( 
    execute immediate stmt 
  ) while rws.c1 < 5
  loop
    dbms_output.put_line ( rws.c1 || ' ' || rws.c2 );
  end loop;
end;
/


select * from v$version;







/* Not working - DON'T SHOW! */
declare
  arr dbms_sql.number_table;
begin
  arr := dbms_sql.number_table ( 
    for i in 1 .. 5 sequence => i 
  );
  for i in indices of arr loop
    dbms_output.put_line ( i || ' is ' || arr(i) );
  end loop;
end;
/

PLS-00681: named association syntax is required
