CREATE OR REPLACE PACKAGE BODY alert_mgr
IS
   g_trc      BOOLEAN := FALSE;
   g_trc_to   VARCHAR2 (10) := 'SCREEN';

   PROCEDURE settrc (on_in IN BOOLEAN, send_to_in IN VARCHAR2 DEFAULT 'SCREEN')
   IS
   BEGIN
      g_trc := on_in;
      g_trc_to := send_to_in;
   END;

   PROCEDURE trc (val IN VARCHAR2)
   IS
   BEGIN
      IF g_trc
      THEN
         IF g_trc_to = 'SCREEN'
         THEN
            DBMS_OUTPUT.put_line (val);
         ELSE
            LOG (val);
         END IF;
      END IF;
   END;

   PROCEDURE trc (val IN BOOLEAN)
   IS
   BEGIN
      trc (
         CASE val WHEN TRUE THEN 'TRUE' WHEN FALSE THEN 'FALSE' ELSE 'NULL' END);
   END;

   PROCEDURE trc (str IN VARCHAR2, val IN BOOLEAN)
   IS
   BEGIN
      trc (
            str
         || ' - '
         || CASE val
               WHEN TRUE THEN 'TRUE'
               WHEN FALSE THEN 'FALSE'
               ELSE 'NULL'
            END);
   END;

   PROCEDURE pl (val IN VARCHAR2)
   IS
   BEGIN
      DBMS_OUTPUT.put_line (val);
   END;

   PROCEDURE bpl (val IN BOOLEAN)
   IS
   BEGIN
      pl (
         CASE val WHEN TRUE THEN 'TRUE' WHEN FALSE THEN 'FALSE' ELSE 'NULL' END);
   END;

   PROCEDURE bpl (str IN VARCHAR2, val IN BOOLEAN)
   IS
   BEGIN
      pl (
            str
         || ' - '
         || CASE val
               WHEN TRUE THEN 'TRUE'
               WHEN FALSE THEN 'FALSE'
               ELSE 'NULL'
            END);
   END;

   PROCEDURE LOG (info_in IN VARCHAR2)
   IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      INSERT INTO alert_log (info)
           VALUES (info_in);

      COMMIT;
   END;

   /* Job management */

   PROCEDURE start_check_alerts_job (every_n_minutes_in IN NUMBER DEFAULT 5)
   IS
   BEGIN
      DBMS_SCHEDULER.create_job (
         job_name          => c_job_name,
         job_type          => 'PLSQL_BLOCK',
         job_action        => 'BEGIN alert_mgr.check_alerts; END;',
         start_date        => SYSDATE,
         repeat_interval   =>    'FREQ=MINUTELY;INTERVAL='
                              || every_n_minutes_in
                              || ';',
         enabled           => FALSE);

      DBMS_SCHEDULER.set_attribute (name        => c_job_name,
                                    attribute   => 'logging_level',
                                    VALUE       => DBMS_SCHEDULER.logging_full);

      DBMS_SCHEDULER.enable (c_job_name);

      UPDATE alert_config
         SET job_wakeup_minutes = every_n_minutes_in;

      resume_checking;
   END;

   PROCEDURE change_check_frequency (every_n_minutes_in IN NUMBER)
   IS
   BEGIN
      DBMS_SCHEDULER.set_attribute (
         name        => c_job_name,
         attribute   => 'repeat_interval',
         VALUE       => 'FREQ=MINUTELY;INTERVAL=' || every_n_minutes_in || ';');

      UPDATE alert_config
         SET job_wakeup_minutes = every_n_minutes_in;
   END;

   PROCEDURE stop_checking
   IS
   BEGIN
      UPDATE alert_config
         SET keep_checking = 'N';
   END;

   PROCEDURE resume_checking
   IS
   BEGIN
      UPDATE alert_config
         SET keep_checking = 'Y';
   END;

   FUNCTION checking_for_alerts
      RETURN BOOLEAN
   IS
      l_keep_checking   alert_config.keep_checking%TYPE;
   BEGIN
      SELECT keep_checking INTO l_keep_checking FROM alert_config;

      RETURN CASE l_keep_checking WHEN 'Y' THEN TRUE ELSE FALSE END;
   END;

   PROCEDURE stop_check_alerts_job
   IS
   BEGIN
      DBMS_SCHEDULER.disable (c_job_name);
   END;

   FUNCTION check_alerts_job_running
      RETURN BOOLEAN
   IS
   BEGIN
      RETURN TRUE;
   END;

   FUNCTION check_alerts_job_running_yn
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN 'Y';
   END;

   PROCEDURE check_alerts_job_status
   IS
   BEGIN
      NULL;
   END;

   PROCEDURE register_alert (
      alert_name_in             IN     VARCHAR2,
      description_in            IN     VARCHAR2 DEFAULT NULL,
      trigger_code_in           IN     VARCHAR2,
      trigger_type_in           IN     VARCHAR2,
      check_frequency_in        IN     VARCHAR2,
      recurrence_frequency_in   IN     VARCHAR2,
      ignore_failures_in        IN     VARCHAR2,
      action_type_in            IN     VARCHAR2,
      action_code_in            IN     VARCHAR2,
      action_email_in           IN     VARCHAR2 DEFAULT NULL,
      is_active_in              IN     VARCHAR2 DEFAULT 'N',
      alert_id_out                 OUT NUMBER)
   IS
   BEGIN
      INSERT INTO alerts (alert_name,
                          description,
                          trigger_code,
                          trigger_type,
                          check_frequency,
                          recurrence_frequency,
                          ignore_failures,
                          action_type,
                          action_code,
                          action_email,
                          is_active)
           VALUES (alert_name_in,
                   description_in,
                   trigger_code_in,
                   trigger_type_in,
                   check_frequency_in,
                   recurrence_frequency_in,
                   ignore_failures_in,
                   action_type_in,
                   action_code_in,
                   action_email_in,
                   is_active_in)
        RETURNING id
             INTO alert_id_out;
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
         UPDATE alerts
            SET description = description_in,
                trigger_code = trigger_code_in,
                trigger_type = trigger_type_in,
                check_frequency = check_frequency_in,
                recurrence_frequency = recurrence_frequency_in,
                ignore_failures = ignore_failures_in,
                action_type = action_type_in,
                action_code = action_code_in,
                action_email = action_email_in,
                is_active = is_active_in
          WHERE alert_name = UPPER (alert_name_in);
   END;

   PROCEDURE register_alert (
      alert_name_in             IN VARCHAR2,
      description_in            IN VARCHAR2 DEFAULT NULL,
      trigger_code_in           IN VARCHAR2,
      trigger_type_in           IN VARCHAR2,
      check_frequency_in        IN VARCHAR2,
      recurrence_frequency_in   IN VARCHAR2,
      ignore_failures_in        IN VARCHAR2,
      action_type_in            IN VARCHAR2,
      action_code_in            IN VARCHAR2,
      action_email_in           IN VARCHAR2 DEFAULT NULL,
      is_active_in              IN VARCHAR2 DEFAULT 'N')
   IS
      alert_id_out   NUMBER;
   BEGIN
      register_alert (alert_name_in             => alert_name_in,
                      description_in            => description_in,
                      trigger_code_in           => trigger_code_in,
                      trigger_type_in           => trigger_type_in,
                      check_frequency_in        => check_frequency_in,
                      recurrence_frequency_in   => recurrence_frequency_in,
                      ignore_failures_in        => ignore_failures_in,
                      action_type_in            => action_type_in,
                      action_code_in            => action_code_in,
                      action_email_in           => action_email_in,
                      is_active_in              => is_active_in,
                      alert_id_out              => alert_id_out);
   END;

   PROCEDURE activate_alert (id_in IN NUMBER)
   IS
   BEGIN
      UPDATE alerts
         SET is_active = 'Y'
       WHERE id = id_in;
   END;

   PROCEDURE deactivate_alert (id_in IN NUMBER)
   IS
   BEGIN
      UPDATE alerts
         SET is_active = 'N'
       WHERE id = id_in;
   END;

   FUNCTION time_to_trigger_alert (check_frequency_in          IN VARCHAR2,
                                   recurring_frequency_in      IN VARCHAR2,
                                   completed_at_in             IN DATE,
                                   completed_successfully_in   IN VARCHAR2,
                                   ignore_failures_in          IN VARCHAR2)
      RETURN BOOLEAN
   IS
      l_overdue       BOOLEAN := FALSE;
      l_check_again   BOOLEAN := FALSE;
      l_check         BOOLEAN := FALSE;

      FUNCTION is_overdue (freq_in IN VARCHAR2)
         RETURN BOOLEAN
      IS
      BEGIN
         RETURN CASE freq_in
                   WHEN c_freq_hourly THEN SYSDATE - completed_at_in >= 1 / 24
                   WHEN c_freq_daily THEN SYSDATE - completed_at_in >= 1
                   ELSE SYSDATE < (completed_at_in + TO_NUMBER (freq_in))
                END;
      END;
   BEGIN
      IF completed_at_in IS NULL
      THEN
         /* Never been run before, so do it the first time. */
         l_check := TRUE;
      ELSE
         /* Might check say every 10 minutes, but you don't want to recur any more frequently than daily. */
         /* If did not complete successfully last time and ignore failures, then see if it should be run again */
         l_overdue := is_overdue (check_frequency_in);
         trc ('check', l_overdue);
         trc ('recur', is_overdue (recurring_frequency_in));

         l_check_again :=
                is_overdue (recurring_frequency_in)
            AND (   completed_successfully_in = 'Y'
                 OR (    completed_successfully_in = 'N'
                     AND ignore_failures_in = 'Y'));

         l_check := l_overdue AND l_check_again;
      END IF;

      RETURN l_check;
   END;

   PROCEDURE execute_alert_action (alert_id_in IN NUMBER)
   IS
      c_start   CONSTANT DATE := SYSDATE;
      l_alert            alerts%ROWTYPE := one_alert (alert_id_in);
      l_success          BOOLEAN := FALSE;
      l_values           DBMS_SQL.varchar2_table;
      l_code             VARCHAR2 (32767) := l_alert.trigger_code;

      PROCEDURE take_action (alert_name_in     IN VARCHAR2,
                             action_type_in    IN VARCHAR2,
                             action_code_in    IN VARCHAR2,
                             action_email_in   IN VARCHAR2)
      IS
      BEGIN
         CASE action_type_in
            WHEN c_action_type_email
            THEN
               apex_mail.send (p_to          => action_email_in,
                               p_from        => 'noreply@oracle.com',
                               p_subj        => 'Alert triggered! ' || alert_name_in,
                               p_body        => 'Take action now!',
                               p_body_html   => '<p>Take action now!</p>');
            WHEN c_action_type_proc
            THEN
               EXECUTE IMMEDIATE
                  'begin ' || RTRIM (action_code_in, ';') || '; end;';
         END CASE;
      END;
   BEGIN
      BEGIN
         CASE
            WHEN l_alert.trigger_type IN (c_trigger_type_select_one_row,
                                          c_trigger_type_select_no_rows,
                                          c_trigger_type_select_mult_rows)
            THEN
               EXECUTE IMMEDIATE l_alert.trigger_code
                  BULK COLLECT INTO l_values;

               l_success :=
                  CASE l_alert.trigger_type
                     WHEN c_trigger_type_select_one_row
                     THEN
                        l_values.COUNT = 1
                     WHEN c_trigger_type_select_no_rows
                     THEN
                        l_values.COUNT = 0
                     WHEN c_trigger_type_select_mult_rows
                     THEN
                        l_values.COUNT > 1
                  END;
            WHEN l_alert.trigger_type = c_trigger_type_return_true
            THEN
               l_code :=
                     'begin :b := '
                  || RTRIM (l_alert.trigger_code, ';')
                  || '; end;';

               EXECUTE IMMEDIATE l_code USING OUT l_success;
            WHEN l_alert.trigger_type = c_trigger_type_return_false
            THEN
               l_code :=
                     'begin :b := '
                  || RTRIM (l_alert.trigger_code, ';')
                  || '; end;';

               EXECUTE IMMEDIATE l_code USING OUT l_success;

               l_success := NOT l_success;
         END CASE;
      EXCEPTION
         WHEN OTHERS
         THEN
            record_result (
               alert_id_in                 => alert_id_in,
               completed_at_in             => SYSDATE,
               completed_successfully_in   => 'N',
               results_in                  =>    'Error executing trigger code: '
                                              || DBMS_UTILITY.format_error_stack
                                              || ' Backtrace: '
                                              || DBMS_UTILITY.format_error_backtrace);
            RETURN;
      END;

      IF l_success
      THEN
         take_action (alert_name_in     => l_alert.alert_name,
                      action_type_in    => l_alert.action_type,
                      action_code_in    => l_alert.action_code,
                      action_email_in   => l_alert.action_email);
      END IF;

      record_result (
         alert_id_in                 => alert_id_in,
         completed_at_in             => SYSDATE,
         completed_successfully_in   => 'Y',
         results_in                  =>    'Trigger code executed: '
                                        || l_code
                                        || CHR (10)
                                        || CHR (10)
                                        || 'Action code executed: '
                                        || l_alert.action_code,
         errors_in                   => NULL);
   EXCEPTION
      WHEN OTHERS
      THEN
         record_result (
            alert_id_in                 => alert_id_in,
            completed_at_in             => SYSDATE,
            completed_successfully_in   => 'N',
            results_in                  =>    'Trigger code executed: '
                                           || l_code
                                           || CHR (10)
                                           || CHR (10)
                                           || 'Action code executed: '
                                           || l_alert.action_code);
   END;

   PROCEDURE check_alerts
   IS
   BEGIN
      IF checking_for_alerts
      THEN
         FOR rec IN (SELECT a.*, ar.completed_at, ar.completed_successfully
                       FROM alerts a, alert_runs ar
                      WHERE a.is_active = 'Y' AND a.last_run_id = ar.id(+))
         LOOP
            /* Is the last run more recent than the recurring frequency?
               If so, do nothing. */
            IF time_to_trigger_alert (rec.check_frequency,
                                      rec.recurrence_frequency,
                                      rec.completed_at,
                                      rec.completed_successfully,
                                      rec.ignore_failures)
            THEN
               execute_alert_action (rec.id);
            END IF;
         END LOOP;
      END IF;
   END;

   /* Add to the alert runs tables and also update the alert row with
      the last run id */
   PROCEDURE record_result (
      alert_id_in                 IN NUMBER,
      completed_at_in             IN DATE DEFAULT SYSDATE,
      completed_successfully_in   IN VARCHAR2 DEFAULT 'Y',
      results_in                  IN VARCHAR2 DEFAULT NULL,
      errors_in                   IN VARCHAR2 DEFAULT NULL)
   IS
      l_id   NUMBER;
   BEGIN
      INSERT INTO alert_runs (alert_id,
                              completed_at,
                              completed_successfully,
                              results,
                              errors)
              VALUES (
                        alert_id_in,
                        completed_at_in,
                        completed_successfully_in,
                        results_in,
                        CASE
                           WHEN completed_successfully_in = 'Y'
                           THEN
                              NULL
                           ELSE
                              NVL (
                                 errors_in,
                                    'Error: '
                                 || DBMS_UTILITY.format_error_stack
                                 || ' Backtrace: '
                                 || DBMS_UTILITY.format_error_backtrace)
                        END)
        RETURNING id
             INTO l_id;

      UPDATE alerts
         SET last_run_id = l_id
       WHERE id = alert_id_in;
   END;

   FUNCTION last_run_for (alert_id_in IN NUMBER)
      RETURN alert_runs%ROWTYPE
   IS
      l_return   alert_runs%ROWTYPE;
   BEGIN
      SELECT ar.*
        INTO l_return
        FROM alert_runs ar, alerts a
       WHERE a.id = alert_id_in AND a.last_run_id = ar.id;

      RETURN l_return;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   FUNCTION one_alert (id_in IN NUMBER)
      RETURN alerts%ROWTYPE
   IS
      l_alert   alerts%ROWTYPE;
   BEGIN
      SELECT *
        INTO l_alert
        FROM alerts
       WHERE id = id_in;

      RETURN l_alert;
   END;
END;
/