CREATE OR REPLACE PACKAGE BODY alert_mgr
IS
   g_trc      BOOLEAN := FALSE;
   g_trc_to   VARCHAR2 (10) := 'SCREEN';

   PROCEDURE settrc (on_in IN BOOLEAN, send_to_in IN VARCHAR2 DEFAULT 'SCREEN')
   IS
   BEGIN
      g_trc := on_in;
      g_trc_to := send_to_in;
   END;

   PROCEDURE trc (val IN VARCHAR2)
   IS
   BEGIN
      IF g_trc
      THEN
         IF g_trc_to = 'SCREEN'
         THEN
            DBMS_OUTPUT.put_line (val);
         ELSE
            log_info (val);
         END IF;
      END IF;
   END;

   PROCEDURE trc (val IN BOOLEAN)
   IS
   BEGIN
      trc (
         CASE val WHEN TRUE THEN 'TRUE' WHEN FALSE THEN 'FALSE' ELSE 'NULL' END);
   END;

   PROCEDURE trc (str IN VARCHAR2, val IN BOOLEAN)
   IS
   BEGIN
      trc (
            str
         || ' - '
         || CASE val
               WHEN TRUE THEN 'TRUE'
               WHEN FALSE THEN 'FALSE'
               ELSE 'NULL'
            END);
   END;

   PROCEDURE pl (val IN VARCHAR2)
   IS
   BEGIN
      DBMS_OUTPUT.put_line (val);
   END;

   PROCEDURE bpl (val IN BOOLEAN)
   IS
   BEGIN
      pl (
         CASE val WHEN TRUE THEN 'TRUE' WHEN FALSE THEN 'FALSE' ELSE 'NULL' END);
   END;

   PROCEDURE bpl (str IN VARCHAR2, val IN BOOLEAN)
   IS
   BEGIN
      pl (
            str
         || ' - '
         || CASE val
               WHEN TRUE THEN 'TRUE'
               WHEN FALSE THEN 'FALSE'
               ELSE 'NULL'
            END);
   END;

   PROCEDURE log_info (info_in IN VARCHAR2)
   IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      INSERT INTO alert_log (info)
           VALUES (info_in);

      COMMIT;
   END;

   FUNCTION recipients_string_to_list (string_in   IN VARCHAR2,
                                       delim_in    IN VARCHAR2 := ',')
      RETURN action_recipients_t
   IS
      l_item       VARCHAR2 (32767);
      l_loc        PLS_INTEGER;
      l_startloc   PLS_INTEGER := 1;
      items_out    action_recipients_t;

      PROCEDURE add_item (item_in IN VARCHAR2)
      IS
      BEGIN
         IF (item_in != delim_in OR item_in IS NULL)
         THEN
            items_out (NVL (items_out.LAST, 0) + 1) := item_in;
         END IF;
      END;
   BEGIN
      IF string_in IS NOT NULL AND delim_in IS NOT NULL
      THEN
         LOOP
            -- Find next delimiter
            l_loc := INSTR (string_in, delim_in, l_startloc);

            IF l_loc = l_startloc                       -- Previous item is NULL
            THEN
               l_item := NULL;
            ELSIF l_loc = 0                       -- Rest of string is last item
            THEN
               l_item := SUBSTR (string_in, l_startloc);
            ELSE
               l_item := SUBSTR (string_in, l_startloc, l_loc - l_startloc);
            END IF;

            add_item (l_item);

            IF l_loc = 0
            THEN
               EXIT;
            ELSE
               l_startloc := l_loc + 1;
            END IF;
         END LOOP;
      END IF;

      RETURN items_out;
   END;

   /* Job management */

   PROCEDURE start_check_alerts_job (every_n_minutes_in IN NUMBER DEFAULT 5)
   IS
   BEGIN
      DBMS_SCHEDULER.create_job (
         job_name          => c_job_name,
         job_type          => 'PLSQL_BLOCK',
         job_action        => 'BEGIN alert_mgr.check_alerts; END;',
         start_date        => SYSDATE,
         repeat_interval   =>    'FREQ=MINUTELY;INTERVAL='
                              || every_n_minutes_in
                              || ';',
         enabled           => FALSE);

      DBMS_SCHEDULER.set_attribute (name        => c_job_name,
                                    attribute   => 'logging_level',
                                    VALUE       => DBMS_SCHEDULER.logging_full);

      DBMS_SCHEDULER.enable (c_job_name);

      UPDATE alert_config
         SET job_wakeup_minutes = every_n_minutes_in;

      resume_checking;
   END;

   PROCEDURE change_check_frequency (every_n_minutes_in IN NUMBER)
   IS
   BEGIN
      DBMS_SCHEDULER.set_attribute (
         name        => c_job_name,
         attribute   => 'repeat_interval',
         VALUE       => 'FREQ=MINUTELY;INTERVAL=' || every_n_minutes_in || ';');

      UPDATE alert_config
         SET job_wakeup_minutes = every_n_minutes_in;
   END;

   PROCEDURE stop_checking
   IS
   BEGIN
      UPDATE alert_config
         SET keep_checking = 'N';
   END;

   PROCEDURE resume_checking
   IS
   BEGIN
      UPDATE alert_config
         SET keep_checking = 'Y';
   END;

   FUNCTION checking_for_alerts
      RETURN BOOLEAN
   IS
      l_keep_checking   alert_config.keep_checking%TYPE;
   BEGIN
      SELECT keep_checking INTO l_keep_checking FROM alert_config;

      RETURN CASE l_keep_checking WHEN 'Y' THEN TRUE ELSE FALSE END;
   END;

   PROCEDURE stop_check_alerts_job
   IS
   BEGIN
      DBMS_SCHEDULER.disable (c_job_name);
   END;

   FUNCTION check_alerts_job_running
      RETURN BOOLEAN
   IS
   BEGIN
      RETURN TRUE;
   END;

   FUNCTION check_alerts_job_running_yn
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN 'Y';
   END;

   PROCEDURE check_alerts_job_status
   IS
   BEGIN
      NULL;
   END;

   PROCEDURE register_email_proc (proc_name_in        IN VARCHAR2,
                                  workspace_name_in   IN VARCHAR2 DEFAULT NULL)
   IS
   BEGIN
      UPDATE alert_config
         SET send_email_proc_name = proc_name_in,
             apex_workspace_name = workspace_name_in;
   END;

   PROCEDURE register_user (email_address_in IN VARCHAR2, id_out OUT NUMBER)
   IS
      l_id   NUMBER;
   BEGIN
      INSERT INTO alert_users (email_address)
           VALUES (email_address_in)
        RETURNING id
             INTO id_out;
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
         /* Already registered */
         NULL;
   END;

   PROCEDURE register_user (email_address_in IN VARCHAR2)
   IS
      l_id   NUMBER;
   BEGIN
      register_user (email_address_in, l_id);
   END;

   FUNCTION register_user (email_address_in IN VARCHAR2)
      RETURN NUMBER
   IS
      l_id   NUMBER;
   BEGIN
      register_user (email_address_in, l_id);
      RETURN l_id;
   END;

   FUNCTION is_query_alert_type (type_in IN VARCHAR2)
      RETURN BOOLEAN
   IS
   BEGIN
      RETURN type_in IN (c_trigtype_select_one_row,
                         c_trigtype_select_no_rows,
                         c_trigtype_select_mult_rows);
   END;

   FUNCTION frequency_description (frequency_in IN VARCHAR2)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN CASE frequency_in
                WHEN 'D' THEN 'Daily'
                WHEN 'H' THEN 'Hourly'
                WHEN '1' THEN 'Every minute'
                WHEN '5' THEN 'Every 5 minutes'
                WHEN '10' THEN 'Every 10 minutes'
                WHEN '15' THEN 'Every 15 minutes'
                WHEN '30' THEN 'Every 30 minutes'
                WHEN '45' THEN 'Every 45 minutes'
                WHEN 'N' THEN 'Never'
             END;
   END;

   PROCEDURE register_alert (
      alert_name_in               IN     VARCHAR2,
      description_in              IN     VARCHAR2 DEFAULT NULL,
      trigger_code_in             IN     VARCHAR2,
      trigger_type_in             IN     VARCHAR2,
      check_frequency_in          IN     VARCHAR2,
      recurrence_frequency_in     IN     VARCHAR2,
      ignore_failures_in          IN     VARCHAR2,
      action_type_in              IN     VARCHAR2 DEFAULT NULL,
      action_code_in              IN     VARCHAR2 DEFAULT NULL,
      send_email_to_in            IN     VARCHAR2 DEFAULT NULL,
      email_subject_in            IN     VARCHAR2 DEFAULT NULL,
      email_body_in               IN     CLOB DEFAULT NULL,
      email_query_in              IN     VARCHAR2 DEFAULT NULL,
      is_active_in                IN     VARCHAR2 DEFAULT 'N',
      owner_name_in               IN     VARCHAR2 DEFAULT NULL,
      table_name_in               IN     VARCHAR2 DEFAULT NULL,
      condition_column1_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator1_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression1_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column2_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator2_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression2_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column3_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator3_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression3_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column4_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator4_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression4_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column5_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator5_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression5_in    IN     VARCHAR2 DEFAULT NULL,
      trigger_key_query_in        IN     VARCHAR2 DEFAULT NULL,
      target_identifier_code_in   IN     VARCHAR2 DEFAULT NULL,
      repeat_for_trigger_key_in   IN     VARCHAR2 DEFAULT NULL,
      repeat_for_target_key_in    IN     VARCHAR2 DEFAULT NULL,
      key_descriptor_code_in      IN     VARCHAR2,
      bind_name_in                IN     VARCHAR2,
      bind_description_in         IN     VARCHAR2,
      bind_datatype_in            IN     VARCHAR2,
      alert_id_out                   OUT NUMBER)
   IS
      PROCEDURE replace_actions (id_in IN INTEGER)
      IS
      BEGIN
         IF action_type_in IS NOT NULL
         THEN
            DELETE FROM alert_actions
                  WHERE alert_id = id_in;

            INSERT INTO alert_actions (alert_id,
                                       action_type,
                                       action_code,
                                       send_email_to,
                                       email_subject,
                                       email_body,
                                       email_query)
                 VALUES (id_in,
                         action_type_in,
                         action_code_in,
                         send_email_to_in,
                         email_subject_in,
                         email_body_in,
                         email_query_in);
         END IF;
      END;
   BEGIN
      INSERT INTO alerts (alert_name,
                          description,
                          trigger_code,
                          trigger_type,
                          check_frequency,
                          recurrence_frequency,
                          ignore_failures,
                          is_active,
                          owner_name,
                          table_name,
                          condition_column1,
                          condition_operator1,
                          condition_expression1,
                          condition_column2,
                          condition_operator2,
                          condition_expression2,
                          condition_column3,
                          condition_operator3,
                          condition_expression3,
                          condition_column4,
                          condition_operator4,
                          condition_expression4,
                          condition_column5,
                          condition_operator5,
                          condition_expression5,
                          trigger_key_query,
                          target_identifier_code,
                          repeat_for_trigger_key,
                          repeat_for_target_key,
                          key_descriptor_code,
                          bind_name,
                          bind_description,
                          bind_datatype)
           VALUES (alert_name_in,
                   description_in,
                   trigger_code_in,
                   trigger_type_in,
                   check_frequency_in,
                   recurrence_frequency_in,
                   ignore_failures_in,
                   is_active_in,
                   owner_name_in,
                   table_name_in,
                   condition_column1_in,
                   condition_operator1_in,
                   condition_expression1_in,
                   condition_column2_in,
                   condition_operator2_in,
                   condition_expression2_in,
                   condition_column3_in,
                   condition_operator3_in,
                   condition_expression3_in,
                   condition_column4_in,
                   condition_operator4_in,
                   condition_expression4_in,
                   condition_column5_in,
                   condition_operator5_in,
                   condition_expression5_in,
                   trigger_key_query_in,
                   target_identifier_code_in,
                   repeat_for_trigger_key_in,
                   repeat_for_target_key_in,
                   key_descriptor_code_in,
                   bind_name_in,
                   bind_description_in,
                   bind_datatype_in)
        RETURNING id
             INTO alert_id_out;

      replace_actions (alert_id_out);
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
            /* TODO Must update all the other columns! */
            UPDATE alerts
               SET description = description_in,
                   trigger_code = trigger_code_in,
                   trigger_type = trigger_type_in,
                   check_frequency = check_frequency_in,
                   recurrence_frequency = recurrence_frequency_in,
                   ignore_failures = ignore_failures_in,
                   is_active = is_active_in,
                   condition_expression1 = condition_expression1_in,
                   table_name = table_name_in,
                   condition_column1 = condition_column1_in,
                   condition_expression3 = condition_expression3_in,
                   condition_expression2 = condition_expression2_in,
                   repeat_for_trigger_key = repeat_for_trigger_key_in,
                   trigger_key_query = trigger_key_query_in,
                   condition_operator5 = condition_operator5_in,
                   condition_operator4 = condition_operator4_in,
                   condition_operator1 = condition_operator1_in,
                   condition_column3 = condition_column3_in,
                   condition_expression5 = condition_expression5_in,
                   condition_column2 = condition_column2_in,
                   condition_expression4 = condition_expression4_in,
                   condition_operator3 = condition_operator3_in,
                   condition_column5 = condition_column5_in,
                   condition_operator2 = condition_operator2_in,
                   condition_column4 = condition_column4_in,
                   owner_name = owner_name_in,
                   bind_description = bind_description_in,
                   bind_name = bind_name_in,
                   repeat_for_target_key = repeat_for_target_key_in,
                   bind_datatype = bind_datatype_in,
                   key_descriptor_code = key_descriptor_code_in,
                   target_identifier_code = target_identifier_code_in
             WHERE alert_name = UPPER (alert_name_in)
         RETURNING id
              INTO alert_id_out;

         replace_actions (alert_id_out);
   END;

   PROCEDURE register_alert (
      alert_name_in               IN VARCHAR2,
      description_in              IN VARCHAR2 DEFAULT NULL,
      trigger_code_in             IN VARCHAR2,
      trigger_type_in             IN VARCHAR2,
      check_frequency_in          IN VARCHAR2,
      recurrence_frequency_in     IN VARCHAR2,
      ignore_failures_in          IN VARCHAR2,
      action_type_in              IN VARCHAR2 DEFAULT NULL,
      action_code_in              IN VARCHAR2 DEFAULT NULL,
      send_email_to_in            IN VARCHAR2 DEFAULT NULL,
      email_subject_in            IN VARCHAR2 DEFAULT NULL,
      email_body_in               IN CLOB DEFAULT NULL,
      email_query_in              IN VARCHAR2 DEFAULT NULL,
      is_active_in                IN VARCHAR2 DEFAULT 'N',
      owner_name_in               IN VARCHAR2 DEFAULT NULL,
      table_name_in               IN VARCHAR2 DEFAULT NULL,
      condition_column1_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator1_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression1_in    IN VARCHAR2 DEFAULT NULL,
      condition_column2_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator2_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression2_in    IN VARCHAR2 DEFAULT NULL,
      condition_column3_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator3_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression3_in    IN VARCHAR2 DEFAULT NULL,
      condition_column4_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator4_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression4_in    IN VARCHAR2 DEFAULT NULL,
      condition_column5_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator5_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression5_in    IN VARCHAR2 DEFAULT NULL,
      trigger_key_query_in        IN VARCHAR2 DEFAULT NULL,
      target_identifier_code_in   IN VARCHAR2 DEFAULT NULL,
      repeat_for_trigger_key_in   IN VARCHAR2 DEFAULT NULL,
      repeat_for_target_key_in    IN VARCHAR2 DEFAULT NULL,
      key_descriptor_code_in      IN VARCHAR2,
      bind_name_in                IN VARCHAR2,
      bind_description_in         IN VARCHAR2,
      bind_datatype_in            IN VARCHAR2)
   IS
      alert_id_out   NUMBER;
   BEGIN
      register_alert (alert_name_in               => alert_name_in,
                      description_in              => description_in,
                      trigger_code_in             => trigger_code_in,
                      trigger_type_in             => trigger_type_in,
                      check_frequency_in          => check_frequency_in,
                      recurrence_frequency_in     => recurrence_frequency_in,
                      ignore_failures_in          => ignore_failures_in,
                      action_type_in              => action_type_in,
                      action_code_in              => action_code_in,
                      send_email_to_in            => send_email_to_in,
                      email_subject_in            => email_subject_in,
                      email_body_in               => email_body_in,
                      email_query_in              => email_query_in,
                      is_active_in                => is_active_in,
                      owner_name_in               => owner_name_in,
                      table_name_in               => table_name_in,
                      condition_column1_in        => condition_column1_in,
                      condition_operator1_in      => condition_operator1_in,
                      condition_expression1_in    => condition_expression1_in,
                      condition_column2_in        => condition_column2_in,
                      condition_operator2_in      => condition_operator2_in,
                      condition_expression2_in    => condition_expression2_in,
                      condition_column3_in        => condition_column3_in,
                      condition_operator3_in      => condition_operator3_in,
                      condition_expression3_in    => condition_expression3_in,
                      condition_column4_in        => condition_column4_in,
                      condition_operator4_in      => condition_operator4_in,
                      condition_expression4_in    => condition_expression4_in,
                      condition_column5_in        => condition_column5_in,
                      condition_operator5_in      => condition_operator5_in,
                      condition_expression5_in    => condition_expression5_in,
                      trigger_key_query_in        => trigger_key_query_in,
                      target_identifier_code_in   => target_identifier_code_in,
                      repeat_for_trigger_key_in   => repeat_for_trigger_key_in,
                      repeat_for_target_key_in    => repeat_for_target_key_in,
                      key_descriptor_code_in      => key_descriptor_code_in,
                      bind_name_in                => bind_name_in,
                      bind_description_in         => bind_description_in,
                      bind_datatype_in            => bind_datatype_in,
                      alert_id_out                => alert_id_out);
   END;

   PROCEDURE register_alert (
      alert_name_in               IN     VARCHAR2,
      description_in              IN     VARCHAR2 DEFAULT NULL,
      trigger_code_in             IN     VARCHAR2,
      trigger_type_in             IN     VARCHAR2,
      check_frequency_in          IN     VARCHAR2,
      recurrence_frequency_in     IN     VARCHAR2,
      ignore_failures_in          IN     VARCHAR2,
      owner_name_in               IN     VARCHAR2 DEFAULT NULL,
      table_name_in               IN     VARCHAR2 DEFAULT NULL,
      condition_column1_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator1_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression1_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column2_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator2_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression2_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column3_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator3_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression3_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column4_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator4_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression4_in    IN     VARCHAR2 DEFAULT NULL,
      condition_column5_in        IN     VARCHAR2 DEFAULT NULL,
      condition_operator5_in      IN     VARCHAR2 DEFAULT NULL,
      condition_expression5_in    IN     VARCHAR2 DEFAULT NULL,
      trigger_key_query_in        IN     VARCHAR2 DEFAULT NULL,
      target_identifier_code_in   IN     VARCHAR2 DEFAULT NULL,
      repeat_for_trigger_key_in   IN     VARCHAR2 DEFAULT NULL,
      repeat_for_target_key_in    IN     VARCHAR2 DEFAULT NULL,
      key_descriptor_code_in      IN     VARCHAR2,
      bind_name_in                IN     VARCHAR2,
      bind_description_in         IN     VARCHAR2,
      bind_datatype_in            IN     VARCHAR2,
      actions_in                  IN     actions_t,
      is_active_in                IN     VARCHAR2 DEFAULT 'N',
      alert_id_out                   OUT NUMBER)
   IS
   BEGIN
      register_alert (alert_name_in               => alert_name_in,
                      description_in              => description_in,
                      trigger_code_in             => trigger_code_in,
                      trigger_type_in             => trigger_type_in,
                      check_frequency_in          => check_frequency_in,
                      recurrence_frequency_in     => recurrence_frequency_in,
                      ignore_failures_in          => ignore_failures_in,
                      is_active_in                => is_active_in,
                      owner_name_in               => owner_name_in,
                      table_name_in               => table_name_in,
                      condition_column1_in        => condition_column1_in,
                      condition_operator1_in      => condition_operator1_in,
                      condition_expression1_in    => condition_expression1_in,
                      condition_column2_in        => condition_column2_in,
                      condition_operator2_in      => condition_operator2_in,
                      condition_expression2_in    => condition_expression2_in,
                      condition_column3_in        => condition_column3_in,
                      condition_operator3_in      => condition_operator3_in,
                      condition_expression3_in    => condition_expression3_in,
                      condition_column4_in        => condition_column4_in,
                      condition_operator4_in      => condition_operator4_in,
                      condition_expression4_in    => condition_expression4_in,
                      condition_column5_in        => condition_column5_in,
                      condition_operator5_in      => condition_operator5_in,
                      condition_expression5_in    => condition_expression5_in,
                      trigger_key_query_in        => trigger_key_query_in,
                      target_identifier_code_in   => target_identifier_code_in,
                      repeat_for_trigger_key_in   => repeat_for_trigger_key_in,
                      repeat_for_target_key_in    => repeat_for_target_key_in,
                      key_descriptor_code_in      => key_descriptor_code_in,
                      bind_name_in                => bind_name_in,
                      bind_description_in         => bind_description_in,
                      bind_datatype_in            => bind_datatype_in,
                      alert_id_out                => alert_id_out);

      DELETE FROM alert_actions
            WHERE alert_id = alert_id_out;

      FORALL indx IN actions_in.FIRST .. actions_in.LAST
         INSERT INTO alert_actions (alert_id,
                                    action_type,
                                    action_code,
                                    send_email_to,
                                    email_subject,
                                    email_body,
                                    email_query)
              VALUES (alert_id_out,
                      actions_in (indx).action_type,
                      actions_in (indx).action_code,
                      actions_in (indx).send_email_to,
                      actions_in (indx).email_subject,
                      actions_in (indx).email_body,
                      actions_in (indx).email_query);
   END;

   PROCEDURE register_alert (
      alert_name_in               IN VARCHAR2,
      description_in              IN VARCHAR2 DEFAULT NULL,
      trigger_code_in             IN VARCHAR2,
      trigger_type_in             IN VARCHAR2,
      check_frequency_in          IN VARCHAR2,
      recurrence_frequency_in     IN VARCHAR2,
      ignore_failures_in          IN VARCHAR2,
      owner_name_in               IN VARCHAR2 DEFAULT NULL,
      table_name_in               IN VARCHAR2 DEFAULT NULL,
      condition_column1_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator1_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression1_in    IN VARCHAR2 DEFAULT NULL,
      condition_column2_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator2_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression2_in    IN VARCHAR2 DEFAULT NULL,
      condition_column3_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator3_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression3_in    IN VARCHAR2 DEFAULT NULL,
      condition_column4_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator4_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression4_in    IN VARCHAR2 DEFAULT NULL,
      condition_column5_in        IN VARCHAR2 DEFAULT NULL,
      condition_operator5_in      IN VARCHAR2 DEFAULT NULL,
      condition_expression5_in    IN VARCHAR2 DEFAULT NULL,
      trigger_key_query_in        IN VARCHAR2 DEFAULT NULL,
      target_identifier_code_in   IN VARCHAR2 DEFAULT NULL,
      repeat_for_trigger_key_in   IN VARCHAR2 DEFAULT NULL,
      repeat_for_target_key_in    IN VARCHAR2 DEFAULT NULL,
      key_descriptor_code_in      IN VARCHAR2,
      bind_name_in                IN VARCHAR2,
      bind_description_in         IN VARCHAR2,
      bind_datatype_in            IN VARCHAR2,
      actions_in                  IN actions_t,
      is_active_in                IN VARCHAR2 DEFAULT 'N')
   IS
      alert_id_out   NUMBER;
   BEGIN
      register_alert (alert_name_in               => alert_name_in,
                      description_in              => description_in,
                      trigger_code_in             => trigger_code_in,
                      trigger_type_in             => trigger_type_in,
                      check_frequency_in          => check_frequency_in,
                      recurrence_frequency_in     => recurrence_frequency_in,
                      ignore_failures_in          => ignore_failures_in,
                      owner_name_in               => owner_name_in,
                      table_name_in               => table_name_in,
                      condition_column1_in        => condition_column1_in,
                      condition_operator1_in      => condition_operator1_in,
                      condition_expression1_in    => condition_expression1_in,
                      condition_column2_in        => condition_column2_in,
                      condition_operator2_in      => condition_operator2_in,
                      condition_expression2_in    => condition_expression2_in,
                      condition_column3_in        => condition_column3_in,
                      condition_operator3_in      => condition_operator3_in,
                      condition_expression3_in    => condition_expression3_in,
                      condition_column4_in        => condition_column4_in,
                      condition_operator4_in      => condition_operator4_in,
                      condition_expression4_in    => condition_expression4_in,
                      condition_column5_in        => condition_column5_in,
                      condition_operator5_in      => condition_operator5_in,
                      condition_expression5_in    => condition_expression5_in,
                      trigger_key_query_in        => trigger_key_query_in,
                      target_identifier_code_in   => target_identifier_code_in,
                      repeat_for_trigger_key_in   => repeat_for_trigger_key_in,
                      repeat_for_target_key_in    => repeat_for_target_key_in,
                      key_descriptor_code_in      => key_descriptor_code_in,
                      bind_name_in                => bind_name_in,
                      bind_description_in         => bind_description_in,
                      bind_datatype_in            => bind_datatype_in,
                      actions_in                  => actions_in,
                      is_active_in                => is_active_in,
                      alert_id_out                => alert_id_out);
   END;

   PROCEDURE follow_alert (alert_id_in     IN NUMBER,
                           user_id_in      IN NUMBER,
                           bind_value_in   IN VARCHAR2)
   IS
   BEGIN
      INSERT INTO alert_follows (alert_id, user_id, bind_value)
           VALUES (alert_id_in, user_id_in, bind_value_in);
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
         /* Already registered */
         NULL;
   END;

   PROCEDURE follow_alert (alert_id_in        IN NUMBER,
                           email_address_in   IN VARCHAR2,
                           bind_value_in      IN VARCHAR2)
   IS
      l_user_id   alert_users.id%TYPE;
   BEGIN
      SELECT id
        INTO l_user_id
        FROM alert_users
       WHERE email_address = UPPER (email_address_in);

      follow_alert (alert_id_in, l_user_id, bind_value_in);
   END;

   PROCEDURE follow_alert (alert_name_in      IN VARCHAR2,
                           email_address_in   IN VARCHAR2,
                           bind_value_in      IN VARCHAR2)
   IS
      l_user_id    alert_users.id%TYPE;
      l_alert_id   alerts.id%TYPE;
   BEGIN
      SELECT u.id, a.id
        INTO l_user_id, l_alert_id
        FROM alert_users u, alerts a
       WHERE     u.email_address = UPPER (email_address_in)
             AND a.alert_name = UPPER (alert_name_in);

      follow_alert (l_alert_id, l_user_id, bind_value_in);
   END;

   PROCEDURE unfollow_alert (alert_id_in IN NUMBER, user_id_in IN NUMBER)
   IS
   BEGIN
      DELETE FROM alert_follows
            WHERE alert_id = alert_id_in AND user_id = user_id_in;
   END;

   PROCEDURE unfollow_alert (alert_name_in      IN VARCHAR2,
                             email_address_in   IN VARCHAR2)
   IS
      l_user_id    alert_users.id%TYPE;
      l_alert_id   alerts.id%TYPE;
   BEGIN
      SELECT u.id, a.id
        INTO l_user_id, l_alert_id
        FROM alert_users u, alerts a
       WHERE     u.email_address = UPPER (email_address_in)
             AND a.alert_name = UPPER (alert_name_in);

      unfollow_alert (l_alert_id, l_user_id);
   END;

   PROCEDURE activate_alert (id_in IN NUMBER)
   IS
   BEGIN
      UPDATE alerts
         SET is_active = 'Y'
       WHERE id = id_in;
   END;

   PROCEDURE deactivate_alert (id_in IN NUMBER)
   IS
   BEGIN
      UPDATE alerts
         SET is_active = 'N'
       WHERE id = id_in;
   END;

   FUNCTION completed_expression (owner_in    IN VARCHAR2,
                                  table_in    IN VARCHAR2,
                                  column_in   IN VARCHAR2,
                                  expr_in     IN VARCHAR2)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN CASE
                WHEN expr_in IS NULL THEN NULL
                ELSE q'[']' || expr_in || q'[']'
             END;
   END;

   FUNCTION constructed_query (alert_in IN alerts%ROWTYPE)
      RETURN VARCHAR2
   IS
      l_query   VARCHAR2 (32767)
         :=    'select 1 from '
            || alert_in.owner_name
            || '.'
            || alert_in.table_name
            || ' where '
            || CASE
                  WHEN alert_in.condition_column1 IS NOT NULL
                  THEN
                        alert_in.condition_column1
                     || ' '
                     || alert_in.condition_operator1
                     || ' '
                     || completed_expression (alert_in.owner_name,
                                              alert_in.table_name,
                                              alert_in.condition_column1,
                                              alert_in.condition_expression1)
               END
            || CASE
                  WHEN alert_in.condition_column2 IS NOT NULL
                  THEN
                        ' and '
                     || alert_in.condition_column2
                     || ' '
                     || alert_in.condition_operator2
                     || ' '
                     || completed_expression (alert_in.owner_name,
                                              alert_in.table_name,
                                              alert_in.condition_column2,
                                              alert_in.condition_expression2)
               END
            || CASE
                  WHEN alert_in.condition_column3 IS NOT NULL
                  THEN
                        ' and '
                     || alert_in.condition_column3
                     || ' '
                     || alert_in.condition_operator3
                     || ' '
                     || completed_expression (alert_in.owner_name,
                                              alert_in.table_name,
                                              alert_in.condition_column3,
                                              alert_in.condition_expression3)
               END
            || CASE
                  WHEN alert_in.condition_column4 IS NOT NULL
                  THEN
                        ' and '
                     || alert_in.condition_column4
                     || ' '
                     || alert_in.condition_operator4
                     || ' '
                     || completed_expression (alert_in.owner_name,
                                              alert_in.table_name,
                                              alert_in.condition_column4,
                                              alert_in.condition_expression4)
               END
            || CASE
                  WHEN alert_in.condition_column5 IS NOT NULL
                  THEN
                        ' and '
                     || alert_in.condition_column5
                     || ' '
                     || alert_in.condition_operator5
                     || ' '
                     || completed_expression (alert_in.owner_name,
                                              alert_in.table_name,
                                              alert_in.condition_column5,
                                              alert_in.condition_expression5)
               END;
   BEGIN
      RETURN l_query;
   END;

   FUNCTION constructed_query (alert_id_in IN NUMBER)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN constructed_query (one_alert (alert_id_in));
   END;

   FUNCTION time_to_trigger_alert (check_frequency_in          IN VARCHAR2,
                                   recurring_frequency_in      IN VARCHAR2,
                                   completed_at_in             IN DATE,
                                   completed_successfully_in   IN VARCHAR2,
                                   ignore_failures_in          IN VARCHAR2)
      RETURN BOOLEAN
   IS
      l_overdue       BOOLEAN := FALSE;
      l_check_again   BOOLEAN := FALSE;
      l_check         BOOLEAN := FALSE;

      FUNCTION is_overdue (freq_in IN VARCHAR2)
         RETURN BOOLEAN
      IS
      BEGIN
         RETURN CASE freq_in
                   WHEN c_freq_never
                   THEN
                      FALSE
                   WHEN c_freq_hourly
                   THEN
                      SYSDATE - completed_at_in >= 1 / 24
                   WHEN c_freq_daily
                   THEN
                      SYSDATE - completed_at_in >= 1
                   ELSE
                      SYSDATE - completed_at_in >=
                         TO_NUMBER (freq_in / (60 * 24))
                END;
      END;
   BEGIN
      IF completed_at_in IS NULL
      THEN
         /* Never been run before, so do it the first time. */
         l_check := TRUE;
      ELSE
         /* Might check say every 10 minutes, but you don't want to recur any more frequently than daily. */
         /* If did not complete successfully last time and ignore failures, then see if it should be run again */
         l_overdue := is_overdue (check_frequency_in);
         trc ('check', l_overdue);
         trc ('recur', is_overdue (recurring_frequency_in));

         l_check_again :=
                is_overdue (recurring_frequency_in)
            AND (   completed_successfully_in = 'Y'
                 OR (    completed_successfully_in = 'N'
                     AND ignore_failures_in = 'Y'));

         l_check := l_overdue AND l_check_again;
      END IF;

      RETURN l_check;
   END;

   FUNCTION is_simple_alert (alert_id_in IN NUMBER)
      RETURN BOOLEAN
   IS
      l_alert   alerts%ROWTYPE := one_alert (alert_id_in);
   BEGIN
      RETURN l_alert.trigger_type <> c_trigtype_bykey;
   END;

   FUNCTION is_simple_alert_yn (alert_id_in IN NUMBER)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN CASE WHEN is_simple_alert (alert_id_in) THEN 'Y' ELSE 'N' END;
   END;

   FUNCTION replaced_text (alert_in       IN alerts%ROWTYPE,
                           user_id_in     IN NUMBER DEFAULT NULL,
                           string_in      IN VARCHAR2,
                           key_value_in   IN VARCHAR2 DEFAULT NULL)
      RETURN VARCHAR2
   IS
      l_alert            alerts%ROWTYPE := alert_in;
      l_alert_follower   alert_follows%ROWTYPE
                            := one_alert_follow (alert_in.id, user_id_in);
   BEGIN
      RETURN REPLACE (
                REPLACE (
                   REPLACE (
                      REPLACE (
                         REPLACE (
                            REPLACE (
                               REPLACE (
                                  REPLACE (
                                     REPLACE (string_in,
                                              c_bind_name_tag,
                                              l_alert.bind_name),
                                     c_bind_datatype_tag,
                                     l_alert.bind_datatype),
                                  l_alert_follower.bind_value),
                               c_bind_description_tag,
                               l_alert.bind_description),
                            c_key_name_tag,
                            /* TODO Must CALL this code with key value. */
                            l_alert.key_descriptor_code),
                         c_key_value_tag,
                         key_value_in),
                      c_check_frequency_text_tag,
                      frequency_description (l_alert.check_frequency)),
                   c_recur_frequency_tag,
                   CASE l_alert.recurrence_frequency
                      WHEN c_freq_hourly THEN 60
                      WHEN c_freq_daily THEN 60 * 24
                      WHEN c_freq_never THEN NULL
                      ELSE                               /* number of minutes */
                          l_alert.recurrence_frequency
                   END),
                c_check_frequency_text_tag,
                CASE l_alert.check_frequency
                   WHEN c_freq_hourly THEN 60
                   WHEN c_freq_daily THEN 60 * 24
                   WHEN c_freq_never THEN NULL
                   ELSE                                  /* number of minutes */
                       l_alert.check_frequency
                END);
   END;

   FUNCTION replaced_text (alert_id_in    IN NUMBER,
                           user_id_in     IN NUMBER DEFAULT NULL,
                           string_in      IN VARCHAR2,
                           key_value_in   IN VARCHAR2 DEFAULT NULL)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN replaced_text (alert_in       => one_alert (alert_id_in),
                            user_id_in     => user_id_in,
                            string_in      => string_in,
                            key_value_in   => key_value_in);
   END;

   FUNCTION is_query (code_in IN VARCHAR2)
      RETURN BOOLEAN
   IS
   BEGIN
      RETURN UPPER (code_in) LIKE 'SELECT%';
   END;

   FUNCTION data_from_code (code_in         IN VARCHAR2,
                            bind_name_in    IN VARCHAR2,
                            bind_value_in   IN VARCHAR2)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN NULL;
   END;

   PROCEDURE execute_alert_actions (alert_id_in IN NUMBER)
   IS
      c_start       CONSTANT DATE := SYSDATE;
      l_alert                alerts%ROWTYPE := one_alert (alert_id_in);
      l_alert_is_triggered   BOOLEAN := FALSE;
      l_code                 VARCHAR2 (32767) := l_alert.trigger_code;

      PROCEDURE take_action (alert_action_in        IN alert_actions%ROWTYPE,
                             trigger_key_value_in   IN VARCHAR2 DEFAULT NULL)
      IS
         l_send_email_proc_name   alert_config.send_email_proc_name%TYPE;
         l_subject                alert_actions.email_subject%TYPE;
         l_body                   alert_actions.email_body%TYPE;
         l_cursor                 SYS_REFCURSOR;
         l_recipients             action_recipients_t;

         /* Index is email. Easy merging of duplicates! */
         TYPE unique_recipients_t IS TABLE OF NUMBER
            INDEX BY VARCHAR2 (1000);

         l_unique_recipients      unique_recipients_t;

         PROCEDURE send_emails_to (recipients_in   IN unique_recipients_t,
                                   subject_in      IN VARCHAR2,
                                   body_in         IN VARCHAR2)
         IS
            l_email_as_index   VARCHAR2 (2000) := recipients_in.FIRST;

            PROCEDURE dynamic_send_email (to_in        IN VARCHAR2,
                                          subject_in   IN VARCHAR2,
                                          body_in      IN VARCHAR2)
            IS
            BEGIN
               EXECUTE IMMEDIATE
                     'BEGIN '
                  || l_send_email_proc_name
                  || '(to_in => :to, subject_in => :sub, body_in => :bod); END;'
                  USING to_in, subject_in, body_in;
            END;
         BEGIN
            WHILE l_email_as_index IS NOT NULL
            LOOP
               dynamic_send_email (to_in        => l_email_as_index,
                                   subject_in   => subject_in,
                                   body_in      => body_in);
               l_email_as_index := recipients_in.NEXT (l_email_as_index);
            END LOOP;
         END;

         FUNCTION merged_recipients_list (
            send_email_to_in   IN VARCHAR2,
            recipients_in      IN action_recipients_t)
            RETURN unique_recipients_t
         IS
            l_recipients          action_recipients_t;
            l_unique_recipients   unique_recipients_t;
         BEGIN
            l_recipients := recipients_string_to_list (send_email_to_in);

            FOR indx IN 1 .. l_recipients.COUNT
            LOOP
               l_unique_recipients (UPPER (l_recipients (indx))) := 1;
            END LOOP;

            FOR indx IN 1 .. recipients_in.COUNT
            LOOP
               l_unique_recipients (UPPER (recipients_in (indx))) := 1;
            END LOOP;

            FOR rec IN (SELECT email_address
                          FROM alert_follows f, alert_users u
                         WHERE f.user_id = u.id AND f.alert_id = alert_id_in)
            LOOP
               l_unique_recipients (UPPER (rec.email_address)) := 1;
            END LOOP;

            /* Now remove any elements from this list if they have already
               receive the alert - and the user said NO MORE */
            IF l_alert.repeat_for_target_key = 'N'
            THEN
               DECLARE
                  l_index   VARCHAR2 (1000);
               BEGIN
                  l_index := l_unique_recipients.FIRST;

                  WHILE l_index IS NOT NULL
                  LOOP
                     IF action_taken (
                           alert_id_in            => alert_id_in,
                           trigger_key_value_in   => trigger_key_value_in,
                           target_key_value_in    => l_unique_recipients (
                                                       l_index))
                     THEN
                        l_unique_recipients.delete (l_index);
                     END IF;

                     l_index := l_unique_recipients.NEXT (l_index);
                  END LOOP;
               END;
            END IF;

            RETURN l_unique_recipients;
         END;
      BEGIN
         log_info (
               'Take action for type-trigger_key_value'
            || alert_action_in.action_type
            || '-'
            || trigger_key_value_in);

         /* If we have a target key value, need to see if it has already been alerted
             and that means no more. */
         IF     l_alert.repeat_for_trigger_key = 'N'
            AND action_taken (alert_id_in            => alert_id_in,
                              trigger_key_value_in   => trigger_key_value_in,
                              target_key_value_in    => NULL)
         THEN
            GOTO post_action;
         END IF;

         IF l_alert.target_identifier_code IS NOT NULL
         THEN
            EXECUTE IMMEDIATE
                  'begin :recipients := '
               || l_alert.target_identifier_code
               || '; end;'
               USING OUT l_recipients, IN trigger_key_value_in;
         END IF;

         CASE alert_action_in.action_type
            WHEN c_action_type_email
            THEN
               SELECT send_email_proc_name
                 INTO l_send_email_proc_name
                 FROM alert_config ac;

               /* THREE ways to specify send-to for the emails:
                  - alert send_email_to list
                  - alert_follows
                  - target list returned above

                  Need to merge into a single list and then test for exclusions if repeat_for_target_key = 'N' */

               l_unique_recipients :=
                  merged_recipients_list (alert_action_in.send_email_to,
                                          l_recipients);

               /* IF there is no email query, then send one. Else.... */
               IF alert_action_in.email_query IS NULL
               THEN
                  send_emails_to (l_unique_recipients,
                                  alert_action_in.email_subject,
                                  alert_action_in.email_body);
               ELSE
                  OPEN l_cursor FOR alert_action_in.email_query;

                  LOOP
                     FETCH l_cursor INTO l_subject, l_body;

                     EXIT WHEN l_cursor%NOTFOUND;

                     send_emails_to (l_unique_recipients, l_subject, l_body);
                  END LOOP;

                  CLOSE l_cursor;
               END IF;
            WHEN c_action_type_proc
            THEN
               EXECUTE IMMEDIATE
                     'BEGIN '
                  || RTRIM (alert_action_in.action_code, ';')
                  || '; end;';
         END CASE;

        <<post_action>>
         /* All done - placeholder for GOTOs that are used to skip action processing */
         NULL;
      END;

      PROCEDURE take_action_by_key
      IS
         l_keys   DBMS_SQL.varchar2_table;
      BEGIN
         EXECUTE IMMEDIATE l_alert.trigger_key_query BULK COLLECT INTO l_keys;

         FOR indx IN 1 .. l_keys.COUNT
         LOOP
            FOR this_action IN (  SELECT *
                                    FROM alert_actions
                                   WHERE alert_id = alert_id_in
                                ORDER BY action_sequence)
            LOOP
               take_action (this_action, l_keys (indx));
            END LOOP;
         END LOOP;
      END;

      FUNCTION time_to_trigger_alert
         RETURN BOOLEAN
      IS
         l_values   DBMS_SQL.varchar2_table;
         l_return   BOOLEAN;
      BEGIN
         CASE
            WHEN l_alert.trigger_type IN (c_trigtype_select_one_row,
                                          c_trigtype_select_no_rows,
                                          c_trigtype_select_mult_rows)
            THEN
               IF l_alert.table_name IS NOT NULL
               THEN
                  l_alert.trigger_code := constructed_query (l_alert);
                  trc ('Constructed query = ' || l_alert.trigger_code);
               END IF;

               EXECUTE IMMEDIATE replaced_text (
                                   alert_in    => l_alert,
                                   string_in   => l_alert.trigger_code)
                  BULK COLLECT INTO l_values;

               l_return :=
                  CASE
                     WHEN l_alert.trigger_type IN (c_trigtype_select_one_row,
                                                   c_trigtype_select_mult_rows)
                     THEN
                           l_values.COUNT = 1
                        OR (    l_alert.trigger_type =
                                   c_trigtype_select_mult_rows
                            AND l_values.COUNT > 1)
                     WHEN l_alert.trigger_type = c_trigtype_select_no_rows
                     THEN
                        l_values.COUNT = 0
                  END;
            WHEN l_alert.trigger_type = c_trigtype_return_true
            THEN
               l_code :=
                  replaced_text (
                     alert_in    => l_alert,
                     string_in   =>    'begin :b := '
                                    || RTRIM (l_alert.trigger_code, ';')
                                    || '; end;');

               EXECUTE IMMEDIATE l_code USING OUT l_return;
            WHEN l_alert.trigger_type = c_trigtype_return_false
            THEN
               l_code :=
                  replaced_text (
                     alert_in    => l_alert,
                     string_in   =>    'begin :b := '
                                    || RTRIM (l_alert.trigger_code, ';')
                                    || '; end;');

               EXECUTE IMMEDIATE l_code USING OUT l_return;

               l_return := NOT l_return;
            WHEN l_alert.trigger_type = c_trigtype_bykey
            THEN
               /* If the target key query returns at least one row, then we have work to do */
               DECLARE
                  l_count   INTEGER;
               BEGIN
                  EXECUTE IMMEDIATE
                        'select count(*) from ('
                     || replaced_text (alert_in    => l_alert,
                                       string_in   => l_alert.trigger_key_query)
                     || ')'
                     INTO l_count;

                  l_return := l_count > 0;
               END;
            ELSE
               record_result (
                  alert_id_in                 => alert_id_in,
                  completed_at_in             => SYSDATE,
                  completed_successfully_in   => 'N',
                  results_in                  =>    'Error executing trigger code: Invalid trigger type: '
                                                 || l_alert.trigger_type);
         END CASE;

         RETURN l_return;
      EXCEPTION
         WHEN OTHERS
         THEN
            record_result (
               alert_id_in                 => alert_id_in,
               completed_at_in             => SYSDATE,
               completed_successfully_in   => 'N',
               results_in                  =>    'Error executing trigger code: '
                                              || DBMS_UTILITY.format_error_stack
                                              || ' Backtrace: '
                                              || DBMS_UTILITY.format_error_backtrace);
            RETURN NULL;
      END;
   BEGIN
      IF time_to_trigger_alert ()
      THEN
         CASE
            WHEN is_simple_alert (alert_id_in)
            THEN
               FOR this_action IN (  SELECT *
                                       FROM alert_actions
                                      WHERE alert_id = alert_id_in
                                   ORDER BY action_sequence)
               LOOP
                  take_action (this_action);
               END LOOP;
            ELSE
               /* Run set of actions for each trigger key value */
               take_action_by_key;
         END CASE;

         record_result (
            alert_id_in                 => alert_id_in,
            completed_at_in             => SYSDATE,
            completed_successfully_in   => 'Y',
            results_in                  => 'Trigger code executed: ' || l_code,
            errors_in                   => NULL);
      END IF;
   EXCEPTION
      WHEN OTHERS
      THEN
         record_result (
            alert_id_in                 => alert_id_in,
            completed_at_in             => SYSDATE,
            completed_successfully_in   => 'N',
            results_in                  => 'Trigger code failed: ' || l_code);
   END;

   PROCEDURE check_alerts
   IS
   BEGIN
      IF checking_for_alerts
      THEN
         FOR rec IN (SELECT a.*, ar.completed_at, ar.completed_successfully
                       FROM alerts a, alert_runs ar
                      WHERE a.is_active = 'Y' AND a.last_run_id = ar.id(+))
         LOOP
            /* Is the last run more recent than the recurring frequency?
               If so, do nothing. */
            IF time_to_trigger_alert (rec.check_frequency,
                                      rec.recurrence_frequency,
                                      rec.completed_at,
                                      rec.completed_successfully,
                                      rec.ignore_failures)
            THEN
               execute_alert_actions (rec.id);
            END IF;
         END LOOP;
      END IF;
   END;

   /* Add to the alert runs tables and also update the alert row with
            the last run id */

   PROCEDURE record_result (
      alert_id_in                 IN NUMBER,
      completed_at_in             IN DATE DEFAULT SYSDATE,
      completed_successfully_in   IN VARCHAR2 DEFAULT 'Y',
      results_in                  IN VARCHAR2 DEFAULT NULL,
      errors_in                   IN VARCHAR2 DEFAULT NULL)
   IS
      l_id   NUMBER;
   BEGIN
      INSERT INTO alert_runs (alert_id,
                              completed_at,
                              completed_successfully,
                              results,
                              errors)
              VALUES (
                        alert_id_in,
                        completed_at_in,
                        completed_successfully_in,
                        results_in,
                        CASE
                           WHEN completed_successfully_in = 'Y'
                           THEN
                              NULL
                           ELSE
                              NVL (
                                 errors_in,
                                    'Error: '
                                 || DBMS_UTILITY.format_error_stack
                                 || ' Backtrace: '
                                 || DBMS_UTILITY.format_error_backtrace)
                        END)
        RETURNING id
             INTO l_id;

      UPDATE alerts
         SET last_run_id = l_id
       WHERE id = alert_id_in;
   END;

   FUNCTION last_run_for (alert_id_in IN NUMBER)
      RETURN alert_runs%ROWTYPE
   IS
      l_return   alert_runs%ROWTYPE;
   BEGIN
      SELECT ar.*
        INTO l_return
        FROM alert_runs ar, alerts a
       WHERE a.id = alert_id_in AND a.last_run_id = ar.id;

      RETURN l_return;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   FUNCTION one_alert (id_in IN NUMBER)
      RETURN alerts%ROWTYPE
   IS
      l_alert   alerts%ROWTYPE;
   BEGIN
      SELECT *
        INTO l_alert
        FROM alerts
       WHERE id = id_in;

      RETURN l_alert;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   FUNCTION one_alert_follow (alert_id_in IN NUMBER, user_id_in IN NUMBER)
      RETURN alert_follows%ROWTYPE
   IS
      l_alert_follow   alert_follows%ROWTYPE;
   BEGIN
      SELECT *
        INTO l_alert_follow
        FROM alert_follows
       WHERE alert_id = alert_id_in AND user_id = user_id_in;

      RETURN l_alert_follow;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   /* Advanced (BYKEY) alert functionality */

   PROCEDURE record_action_taken (
      alert_run_id_in        IN NUMBER,
      trigger_key_value_in   IN VARCHAR2,
      target_key_value_in    IN VARCHAR2 DEFAULT NULL)
   IS
   BEGIN
      INSERT
        INTO alert_recipients (alert_run_id, trigger_key_value, target_key_value)
      VALUES (alert_run_id_in, trigger_key_value_in, target_key_value_in);
   END;

   FUNCTION action_taken (alert_id_in            IN NUMBER,
                          trigger_key_value_in   IN VARCHAR2,
                          target_key_value_in    IN VARCHAR2 DEFAULT NULL)
      RETURN BOOLEAN
   IS
      l_count   INTEGER;
   BEGIN
      SELECT COUNT (*)
        INTO l_count
        FROM alert_recipients ar, alert_runs r
       WHERE     r.alert_id = alert_id_in
             AND ar.alert_run_id = r.id
             AND trigger_key_value = trigger_key_value_in
             AND ( (   target_key_value = target_key_value_in
                    OR (    target_key_value IS NULL
                        AND target_key_value_in IS NULL)));

      RETURN l_count > 0;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN FALSE;
   END;

   FUNCTION action_taken_yn (alert_id_in            IN NUMBER,
                             trigger_key_value_in   IN VARCHAR2,
                             target_key_value_in    IN VARCHAR2 DEFAULT NULL)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN CASE
                WHEN action_taken (alert_id_in,
                                   trigger_key_value_in,
                                   target_key_value_in)
                THEN
                   'Y'
                ELSE
                   'N'
             END;
   END;
END;
/