DECLARE
   /*
   DEV:        https://gxpbt.oracle.com/oalcrm/service/mkt/promos/oh/whitelist
   UAT:         https://gxpdt.oracle.com/oalcrm/service/mkt/promos/oh/whitelist
   PROD:        https://gxpap.oracle.com/oalcrm/service/mkt/promos/oh/whitelist

   */
   c_proxy    CONSTANT VARCHAR2 (100) := 'appoci-proxy01-vip.oraclevcn.com';
   c_wallet   CONSTANT VARCHAR2 (100)
      := 'file:/var/opt/oracle/dbaas_acfs/AAPPSCDB/apex_wallet' ;

   l_request           VARCHAR2 (32767);
   l_url               VARCHAR2 (32767)
      := 'https://gxpbt.oracle.com/oalcrm/service/mkt/promos/oh/whitelist';
   l_clob              CLOB;
   l_http_method       VARCHAR2 (10) := 'POST';
   l_response_json     apex_json.t_values;
   l_status            VARCHAR2 (32767);
   
   FUNCTION generate_jwt
      RETURN VARCHAR2
   AS
      l_header            VARCHAR2 (32767);
      l_payload           VARCHAR2 (32767);
      l_result            VARCHAR2 (32767);
      l_signature         VARCHAR2 (32767);
      c_key      CONSTANT VARCHAR2 (100) := '1SECmc0yQ0u2o8Vs8d6r0A';
      c_secret   CONSTANT VARCHAR2 (100)
                             := '1mIOwjciR08KjJizHJR5Fucn0QroHDC5svRP' ;

      FUNCTION to_seconds_since_epoch (p_tstz IN TIMESTAMP WITH TIME ZONE)
         RETURN PLS_INTEGER
      IS
      BEGIN
         RETURN   (  CAST ( (p_tstz AT TIME ZONE 'UTC') AS DATE)
                   - DATE '1970-01-01')
                * 86400;
      END to_seconds_since_epoch;

      FUNCTION to_url (p_str IN VARCHAR2)
         RETURN VARCHAR2
      IS
      BEGIN
         RETURN TRANSLATE (p_str, UNISTR ('+/=\000a\000d'), '-_');
      END to_url;

      FUNCTION base64url_encode (p_str IN VARCHAR2)
         RETURN VARCHAR2
      IS
      BEGIN
         RETURN to_url (
                   sys.UTL_RAW.cast_to_varchar2 (
                      sys.UTL_ENCODE.base64_encode (
                         sys.UTL_RAW.cast_to_raw (p_str))));
      END base64url_encode;

      FUNCTION get_hs256_signature (p_header_and_payload   IN VARCHAR2,
                                    p_signature_key        IN RAW)
         RETURN VARCHAR2
      IS
      BEGIN
         RETURN to_url (
                   sys.UTL_RAW.cast_to_varchar2 (
                      sys.UTL_ENCODE.base64_encode (
                         sys.DBMS_CRYPTO.mac (
                            src   => sys.UTL_RAW.cast_to_raw (
                                       p_header_and_payload),
                            typ   => sys.DBMS_CRYPTO.hmac_sh256,
                            key   => p_signature_key))));
      END get_hs256_signature;
   BEGIN
      l_header := '{"alg":"HS256","typ":"JWT"}';
      l_header := base64url_encode (l_header);

      l_payload :=
            '{'
         || '  "iss": "'
         || c_key
         || '",'
         || '  "exp": '
         || to_seconds_since_epoch (
               SYSTIMESTAMP + NUMTODSINTERVAL (30, 'second'))
         || ','
         || '  "iat": '
         || to_seconds_since_epoch (SYSTIMESTAMP)
         || '}';
      l_payload := base64url_encode (l_payload);

      l_result := l_header || '.' || l_payload;

      l_signature :=
         get_hs256_signature (
            p_header_and_payload   => l_result,
            p_signature_key        => sys.UTL_RAW.cast_to_raw (c_secret));

      RETURN (l_result || '.' || l_signature);
   END generate_jwt;

   PROCEDURE set_request_headers
   IS
   BEGIN
      apex_web_service.g_request_headers (1).name := 'Content-Type';
      apex_web_service.g_request_headers (1).VALUE := 'application/json';
      apex_web_service.g_request_headers (2).name := 'Authorization';
      apex_web_service.g_request_headers (2).VALUE :=
         'Bearer ' || generate_jwt ();
   END;
BEGIN
   apex_json.initialize_clob_output;
   apex_json.open_object ();

   apex_json.write ('email', 'steven.feuerstein@oracle.com');
   apex_json.write ('programId', '123');
   apex_json.close_object;

   l_request := apex_json.get_clob_output ();

   set_request_headers;

   l_clob :=
      apex_web_service.make_rest_request (p_http_method      => l_http_method,
                                          p_url              => l_url,
                                          p_body             => l_request,
                                          p_wallet_path      => null, -- c_wallet,
                                          p_wallet_pwd       => NULL,
                                          p_proxy_override   => null); -- c_proxy);

   apex_json.parse (l_response_json, l_clob);

   l_status :=
      apex_json.get_number (p_path => 'status', p_values => l_response_json);
END;
/
