create or replace function tmr return number is
   l_return number;
begin
   select 1
     into l_return
     from hr.employees;
   return l_return;
end tmr;
/

/* What will I see? */

select last_name,
       tmr
  from hr.employees
/

create or replace function ndf return number is
   l_return number;
begin
   select 1
     into l_return
     from dual
    where 1 = 0;
   return l_return;
end ndf;
/

/* What will I see? */

select last_name,
       ndf
  from hr.employees
/

/* Let's look at some timings */

create or replace function fullname (
   l  in  varchar2,
   f  in  varchar2
) return varchar2 is
begin
   return l
          || ','
          || f;
end fullname;
/

create or replace function max_salary (
   job_in in varchar2
) return number is
begin
   return 25000;
end max_salary;
/

create or replace package fullname_pkg is
   function fullname (
      l  in  varchar2,
      f  in  varchar2
   ) return varchar2;

   function max_salary (
      job_in in varchar2
   ) return number;
end fullname_pkg;
/

create or replace package body fullname_pkg is
   function fullname (
      l  in  varchar2,
      f  in  varchar2
   ) return varchar2 is
   begin
      return l
             || ','
             || f;
   end fullname;

   function max_salary (
      job_in in varchar2
   ) return number is
   begin
      return 25000;
   end max_salary;
end fullname_pkg;
/

declare
   l_counter     pls_integer := 1000;
   l_start_time  number;
   l_name        varchar2(32767);

   procedure show_elapsed (
      string_in in varchar2
   ) is
   begin
      DBMS_OUTPUT.put_line('Elapsed time for "'
                           || string_in
                           || '" = '
                           || TO_CHAR((DBMS_UTILITY.get_cpu_time - l_start_time) / 100)
                           || ' seconds.');
   end show_elapsed;
begin
   DBMS_OUTPUT.put_line(l_counter || ' Iterations');
   --
   l_start_time  := DBMS_UTILITY.get_cpu_time;
   for i in 1..l_counter loop
      select last_name
             || ','
             || first_name
        into l_name
        from hr.employees
       where employee_id = 137;
   end loop;

   show_elapsed('No function in select list');
   --
   l_start_time  := DBMS_UTILITY.get_cpu_time;
   for i in 1..l_counter loop
      select fullname(last_name, first_name)
        into l_name
        from hr.employees
       where employee_id = 137;
   end loop;

   show_elapsed('Schema-level function in select list');
   --
   l_start_time  := DBMS_UTILITY.get_cpu_time;
   for i in 1..l_counter loop
      select fullname_pkg.fullname(last_name, first_name)
        into l_name
        from hr.employees
       where employee_id = 137;
   end loop;

   show_elapsed('Packaged function in select list');
   --
   l_start_time  := DBMS_UTILITY.get_cpu_time;
   for i in 1..l_counter loop
      select min(salary)
        into l_name
        from hr.employees
       where salary <= 25000;
   end loop;

   show_elapsed('No function in where clause');
   --
   l_start_time  := DBMS_UTILITY.get_cpu_time;
   for i in 1..l_counter loop
      select min(salary)
        into l_name
        from hr.employees
       where salary <= max_salary(job_id);
   end loop;

   show_elapsed('Schema-level function in where clause');
   --
   l_start_time  := DBMS_UTILITY.get_cpu_time;
   for i in 1..l_counter loop
      select min(salary)
        into l_name
        from hr.employees
       where salary <= fullname_pkg.max_salary(job_id);
   end loop;

   show_elapsed('Packaged function in where clause');
/*
50000 iterations

Elapsed time for "No function in select list" = 1.87 seconds.
Elapsed time for "Schema-level function in select list" = 2.97 seconds.
Elapsed time for "Packaged function in select list" = 3.15 seconds.
Elapsed time for "No function in where clause" = 4.01 seconds.
Elapsed time for "Schema-level function in where clause" = 31.06 seconds.
Elapsed time for "Packaged function in where clause" = 33 seconds.

1000 iterations

*/
end;
/