CREATE OR REPLACE PROCEDURE gen_table_ddl (
   entity_in     IN VARCHAR2,
   entities_in   IN VARCHAR2 DEFAULT NULL,
   add_fky_in    IN BOOLEAN DEFAULT TRUE,
   prefix_in     IN VARCHAR2 DEFAULT NULL,
   in_apex_in    IN BOOLEAN DEFAULT FALSE)
IS
   c_table_name    CONSTANT VARCHAR2 (100)
      := prefix_in || NVL (entities_in, entity_in || 's') ;

   c_pkycol_name   CONSTANT VARCHAR2 (100) := entity_in || '_ID';

   c_user_code     CONSTANT VARCHAR2 (100)
      := CASE
            WHEN in_apex_in THEN 'NVL (v (''APP_USER''), USER)'
            ELSE 'USER'
         END ;

   PROCEDURE pl (str_in                   IN VARCHAR2,
                 indent_in                IN INTEGER DEFAULT 3,
                 num_newlines_before_in   IN INTEGER DEFAULT 0)
   IS
   BEGIN
      FOR indx IN 1 .. num_newlines_before_in
      LOOP
         DBMS_OUTPUT.put_line ('');
      END LOOP;

      DBMS_OUTPUT.put_line (LPAD (' ', indent_in) || str_in);
   END;
BEGIN
   pl ('CREATE TABLE ' || c_table_name || '(', 0);
   pl (c_pkycol_name || ' INTEGER NOT NULL,');
   pl ('created_by VARCHAR2 (132 BYTE) NOT NULL,');
   pl ('changed_by VARCHAR2 (132 BYTE) NOT NULL,');
   pl ('created_on DATE NOT NULL,');
   pl ('changed_on DATE NOT NULL');
   pl (');');

   pl ('CREATE SEQUENCE ' || c_table_name || '_SEQ;', 0, 1);
   pl (
         'CREATE UNIQUE INDEX '
      || c_table_name
      || ' ON '
      || c_table_name
      || '('
      || c_pkycol_name
      || ');',
      0,
      1);
   pl (
         'CREATE OR REPLACE TRIGGER '
      || c_table_name
      || '_bir 
      BEFORE INSERT ON '
      || c_table_name,
      0,
      1);
   pl ('FOR EACH ROW DECLARE', 3);
   pl ('BEGIN', 3);
   pl ('IF :new.' || c_pkycol_name || ' IS NULL', 6);
   pl (
         'THEN :new.'
      || c_pkycol_name
      || ' := '
      || c_table_name
      || '_seq.NEXTVAL; END IF;',
      6);

   pl (':new.created_on := SYSDATE;', 6);
   pl (':new.created_by := ' || c_user_code || ';', 6);
   pl (':new.changed_on := SYSDATE;', 6);
   pl (':new.changed_by := ' || c_user_code || ';', 6);
   pl ('END ' || c_table_name || '_bir;', 3);

   pl ('CREATE OR REPLACE TRIGGER ' || c_table_name || '_bur', 0, 1);
   pl ('BEFORE UPDATE ON ' || c_table_name || ' FOR EACH ROW', 3);
   pl ('DECLARE', 3);
   pl ('BEGIN', 3);
   pl (':new.changed_on := SYSDATE;', 6);
   pl (':new.changed_by := ' || c_user_code || ';', 6);
   pl ('END ' || c_table_name || '_bur;', 3);

   pl ('ALTER TABLE ' || c_table_name || ' ADD 
      (CONSTRAINT ' || c_table_name,
       0,
       1);
   pl (
         'PRIMARY KEY ('
      || c_pkycol_name
      || ') 
       USING INDEX '
      || c_table_name
      || ' ENABLE VALIDATE);',
      3);

   IF add_fky_in
   THEN
      pl (
            'ALTER TABLE '
         || c_table_name
         || ' ADD (CONSTRAINT fk_'
         || c_table_name,
         0,
         1);
      pl ('FOREIGN KEY (REPLACE_id) 
     REFERENCES qdb_REPLACE (REPLACE_id)', 3);
      pl ('ON DELETE CASCADE ENABLE VALIDATE);', 3);
   END IF;
END;
/

BEGIN
   gen_table_ddl (entity_in     => 'EMPLOYEE',
                  entities_in   => NULL,
                  add_fky_in    => TRUE,
                  prefix_in     => 'HR_',
                  in_apex_in    => FALSE);
END;
/


BEGIN
   gen_table_ddl (entity_in     => 'COUNTRY',
                  entities_in   => 'COUNTRIES',
                  add_fky_in    => FALSE,
                  prefix_in     => 'WRLD$',
                  in_apex_in    => TRUE);
END;
/