CREATE TABLE emp
(
   empno      NUMBER (4) NOT NULL,
   ename      CHAR (10),
   job        CHAR (9),
   mgr        NUMBER (4),
   hiredate   DATE,
   sal        NUMBER (7, 2),
   comm       NUMBER (7, 2),
   deptno     NUMBER (2) NOT NULL
);

BEGIN
   INSERT INTO emp
        VALUES (7839,
                'KING',
                'PRESIDENT',
                NULL,
                '17-NOV-81',
                5000,
                NULL,
                10);

   INSERT INTO emp
        VALUES (7698,
                'BLAKE',
                'MANAGER',
                7839,
                '1-MAY-81',
                2850,
                NULL,
                30);

   INSERT INTO emp
        VALUES (7782,
                'CLARK',
                'MANAGER',
                7839,
                '9-JUN-81',
                2450,
                NULL,
                10);

   INSERT INTO emp
        VALUES (7566,
                'JONES',
                'MANAGER',
                7839,
                '2-APR-81',
                2975,
                NULL,
                20);

   INSERT INTO emp
        VALUES (7654,
                'MARTIN',
                'SALESMAN',
                7698,
                '28-SEP-81',
                1250,
                1400,
                30);

   INSERT INTO emp
        VALUES (7499,
                'ALLEN',
                'SALESMAN',
                7698,
                '20-FEB-81',
                1600,
                300,
                30);

   INSERT INTO emp
        VALUES (7844,
                'TURNER',
                'SALESMAN',
                7698,
                '8-SEP-81',
                1500,
                0,
                30);

   INSERT INTO emp
        VALUES (7900,
                'JAMES',
                'CLERK',
                7698,
                '3-DEC-81',
                950,
                NULL,
                30);

   INSERT INTO emp
        VALUES (7521,
                'WARD',
                'SALESMAN',
                7698,
                '22-FEB-81',
                1250,
                500,
                30);

   INSERT INTO emp
        VALUES (7902,
                'FORD',
                'ANALYST',
                7566,
                '3-DEC-81',
                3000,
                NULL,
                20);

   INSERT INTO emp
        VALUES (7369,
                'SMITH',
                'CLERK',
                7902,
                '17-DEC-80',
                800,
                NULL,
                20);

   INSERT INTO emp
        VALUES (7788,
                'SCOTT',
                'ANALYST',
                7566,
                '09-DEC-82',
                3000,
                NULL,
                20);

   INSERT INTO emp
        VALUES (7876,
                'ADAMS',
                'CLERK',
                7788,
                '12-JAN-83',
                1100,
                NULL,
                20);

   INSERT INTO emp
        VALUES (7934,
                'MILLER',
                'CLERK',
                7782,
                '23-JAN-82',
                1300,
                NULL,
                10);

   COMMIT;
END;
/


ALTER TABLE emp
   ADD shoesize NUMBER;

ALTER TABLE emp
   DROP COLUMN shoesize;


-- needs dba, change of queries, downtime, etc
-- instead people add 'Flexfields' right from the beginning:
-- 10 varchar2 fields
-- 10 number fields
-- 5 dates, 5 timestamps
-- indicator column



ALTER TABLE emp
   ADD flex CLOB CHECK (flex IS JSON);

SELECT e.flex.shoesize
  FROM emp e;

SELECT e.flex.skills
  FROM emp e;

SELECT e.flex.photourl
  FROM emp e;

UPDATE emp
   SET flex = 'xyz'
 WHERE empno = 7934;

UPDATE emp
   SET flex = '{"shoeSize":12, "skills":["oracle", "c", "java"]}'
 WHERE empno = 7934;

UPDATE emp
   SET flex = '{"shoeSize":10.5, "skills":["hiring","firing"]}'
 WHERE empno = 7902;

COMMIT;

SELECT e.flex.shoesize
  FROM emp e;

SELECT e.flex.shoeSize
  FROM emp e;

SELECT e.flex.skills
  FROM emp e;

SELECT e.flex.skills[0]
  FROM emp e;


-- expose as views

CREATE VIEW empv1
AS
   SELECT empno,
          ename,
          job,
          mgr,
          hiredate,
          sal,
          comm,
          deptno,
          emp.flex.shoesize "SHOESIZE",
          emp.flex.skills "SKILLS"
     FROM emp emp;

DESC empv1;

SELECT ename, shoesize, skills
  FROM empv1
 WHERE shoesize > 11;


-- skills is still a json array
-- unnest it using json_tables

SELECT empno,
       ename,
       job,
       mgr,
       hiredate,
       sal,
       comm,
       deptno,
       emp.flex.shoesize "SHOESIZE",
       jt.skill
  FROM emp emp, json_table(emp.flex , '$.skills[*]' columns (
         "SKILL" varchar2(20) path '$')) jt;

SELECT empno,
       ename,
       job,
       mgr,
       hiredate,
       sal,
       comm,
       deptno,
       emp.flex.shoesize "SHOESIZE",
       jt.skill
  FROM emp emp LEFT OUTER JOIN json_table(emp.flex , '$.skills[*]' columns (
         "SKILL" varchar2(20) path '$')) jt
       ON (1=1);


-- could be exposed as view as well

CREATE VIEW empv2
AS
   SELECT empno,
          ename,
          job,
          mgr,
          hiredate,
          sal,
          comm,
          deptno,
          emp.flex.shoesize "SHOESIZE",
          jt.skill
     FROM emp emp LEFT OUTER JOIN json_table(emp.flex , '$.skills[*]' columns (
         "SKILL" varchar2(20) path '$')) jt
       ON (1=1);

SELECT ename
  FROM empv2
 WHERE skill = 'oracle';



-- pl/sql procedure to set/get a field

CREATE OR REPLACE FUNCTION getfield (flex IN CLOB, name IN VARCHAR)
   RETURN VARCHAR
IS
   val   VARCHAR2 (4000);
   o     json_object_t;
BEGIN
   IF flex IS NULL
   THEN
      RETURN NULL;
   END IF;

   o := NEW json_object_t (flex);
   val := o.get_string (name);
   RETURN val;
END;
/

SELECT ename, getfield (flex, 'shoeSize')
  FROM emp
 WHERE flex IS NOT NULL
/

CREATE OR REPLACE FUNCTION setfield (flex    IN CLOB,
                                     name    IN VARCHAR,
                                     VALUE   IN VARCHAR)
   RETURN VARCHAR2
IS
   o   json_object_t;
   s   VARCHAR2 (4000);
BEGIN
   IF flex IS NULL
   THEN
      o := NEW json_object_t;
   ELSE
      o := NEW json_object_t (flex);
   END IF;

   o.put (name, VALUE);
   s := o.to_string;
   RETURN s;
END;
/

UPDATE emp
   SET flex = setfield (flex, 'id', empno);