CREATE SEQUENCE execution_trace_seq
/

CREATE TABLE execution_trace
(
   "LOG_ID"           NUMBER (*, 0),
   "CONTEXT"          VARCHAR2 (500),
   "TEXT"             VARCHAR2 (4000),
   "CALLSTACK"        VARCHAR2 (4000),
   "CREATED_ON"       DATE,
   "CREATED_BY"       VARCHAR2 (100),
   "CHANGED_ON"       DATE,
   "CHANGED_BY"       VARCHAR2 (100),
   "FULL_TEXT"        CLOB,
   "FULL_CALLSTACK"   CLOB,
   PRIMARY KEY ("LOG_ID") USING INDEX ENABLE
)
/

CREATE OR REPLACE EDITIONABLE TRIGGER execution_trace_bir
   BEFORE INSERT
   ON execution_trace
   FOR EACH ROW
DECLARE
BEGIN
   IF :new.log_id IS NULL
   THEN
      :new.log_id := execution_trace_seq.NEXTVAL;
   END IF;

   :new.created_on := SYSDATE;
   :new.created_by := USER;
   :new.changed_on := SYSDATE;
   :new.changed_by := USER;
END;
/

ALTER TRIGGER execution_trace_bir ENABLE
/

CREATE TABLE "SYSTEM_LOG"
(
   "SYSTEM_LOG_ID"   NUMBER (38, 0) NOT NULL ENABLE,
   "LOGTYPE"         VARCHAR2 (20) NOT NULL ENABLE,
   "ACTION"          VARCHAR2 (100) NOT NULL ENABLE,
   "TITLE"           VARCHAR2 (1000) NOT NULL ENABLE,
   "DETAILS"         VARCHAR2 (4000),
   "CALLSTACK"       VARCHAR2 (4000),
   "ERRORSTACK"      VARCHAR2 (4000),
   "ERRORTRACE"      VARCHAR2 (4000),
   "CREATED_BY"      VARCHAR2 (132) NOT NULL ENABLE,
   "CHANGED_BY"      VARCHAR2 (132) NOT NULL ENABLE,
   "CREATED_ON"      DATE NOT NULL ENABLE,
   "CHANGED_ON"      DATE NOT NULL ENABLE,
   "APP_ID"          NUMBER (*, 0),
   "SESSION_ID"      NUMBER (*, 0),
   "PAGE_ID"         NUMBER (*, 0),
   "FULL_DETAILS"    CLOB,
   CONSTRAINT "PKY_SYSTEM_LOG" PRIMARY KEY
      ("SYSTEM_LOG_ID")
      USING INDEX ENABLE
)
ENABLE ROW MOVEMENT
/

CREATE INDEX "SYSTEM_LOG_IDX$$_263400BD"
   ON "SYSTEM_LOG" ("CREATED_ON")
/

CREATE OR REPLACE EDITIONABLE TRIGGER "SYSTEM_LOG_BIR"
   BEFORE INSERT
   ON system_log
   FOR EACH ROW
DECLARE
BEGIN
   IF :new.system_log_id IS NULL
   THEN
      :new.system_log_id := system_log_seq.NEXTVAL;
   END IF;

   :new.created_on := SYSDATE;
   :new.created_by := USER;
   :new.changed_on := SYSDATE;
   :new.changed_by := USER;
END system_log_bir;
/

ALTER TRIGGER  "SYSTEM_LOG_BIR" ENABLE
/

CREATE OR REPLACE EDITIONABLE TRIGGER "SYSTEM_LOG_BUR"
   BEFORE UPDATE
   ON system_log
   FOR EACH ROW
BEGIN
   :new.changed_on := SYSDATE;
   :new.changed_by := config.apex_user;
   :new.created_on := NVL (:old.created_on, SYSDATE);
END;
/

ALTER TRIGGER  "SYSTEM_LOG_BUR" ENABLE
/