CREATE OR REPLACE PACKAGE ora_logger
   AUTHID DEFINER
IS
   SUBTYPE maxvarchar2 IS VARCHAR2 (32767);

   TYPE maxvarchar2_aat IS TABLE OF VARCHAR2 (32767)
      INDEX BY PLS_INTEGER;

   /* For tracing */
   c_to_screen         CONSTANT VARCHAR2 (1) := 'S';
   c_to_table          CONSTANT VARCHAR2 (1) := 'T';

   /* Used by display header and pl */
   g_output                     DBMS_SQL.varchar2a;

   c_twolines          CONSTANT CHAR (2) := CHR (10) || CHR (10);

   /* 3.1 special exception for reraise, avoid multiple loggings */
   e_reraise                    EXCEPTION;
   c_reraise_errnum    CONSTANT INTEGER := -20999;
   PRAGMA EXCEPTION_INIT (e_reraise, -20999);

   /* System log for error and other reporting */
   
   PROCEDURE syslog (logtype_in   IN VARCHAR2,
                     action_in    IN VARCHAR2,
                     title_in     IN VARCHAR2,
                     details_in   IN VARCHAR2);

   PROCEDURE syslog_error (action_in         IN     VARCHAR2,
                           title_in          IN     VARCHAR2,
                           details_in        IN     VARCHAR2,
                           raise_error_in    IN     BOOLEAN DEFAULT FALSE,
                           message_only_in   IN     BOOLEAN DEFAULT FALSE,
                           user_message_in   IN     VARCHAR2 DEFAULT NULL,
                           log_id_out           OUT NUMBER);

   PROCEDURE syslog_error (action_in         IN VARCHAR2,
                           title_in          IN VARCHAR2,
                           details_in        IN VARCHAR2,
                           raise_error_in    IN BOOLEAN DEFAULT FALSE,
                           message_only_in   IN BOOLEAN DEFAULT FALSE,
                           user_message_in   IN VARCHAR2 DEFAULT NULL);

   PROCEDURE syslog_information (action_in    IN VARCHAR2,
                                 title_in     IN VARCHAR2,
                                 details_in   IN VARCHAR2 DEFAULT NULL);

   PROCEDURE syslog_warning (action_in    IN VARCHAR2,
                             title_in     IN VARCHAR2,
                             details_in   IN VARCHAR2);

   PROCEDURE syslog_assert (condition_in IN BOOLEAN, text_in IN VARCHAR2);

   PROCEDURE send_syslog_report (from_in      IN DATE DEFAULT NULL,
                                 to_in        IN DATE DEFAULT NULL,
                                 send_to_in   IN VARCHAR2 DEFAULT NULL);

   PROCEDURE send_syslog_report_last_hour;

   /* Trace functionality */

   PROCEDURE turn_on_trace (
      context_filter_in       IN VARCHAR2 DEFAULT NULL,
      this_session_only_in    IN BOOLEAN DEFAULT FALSE,
      callstack_contains_in   IN VARCHAR2 DEFAULT NULL,
      send_trace_to_in        IN VARCHAR2 DEFAULT c_to_table,
      max_log_entries_in      IN INTEGER DEFAULT NULL);

   PROCEDURE turn_off_trace;

   FUNCTION trace_is_on (context_filter_in IN VARCHAR2 DEFAULT NULL)
      RETURN BOOLEAN;

   PROCEDURE trace_activity (context_in IN VARCHAR2, data_in IN VARCHAR2);

   PROCEDURE trace_activity (
      context_in   IN VARCHAR2,
      data_in      IN DATE,
      format_in    IN VARCHAR2 DEFAULT 'YYYY-MM-DD HH24:MI:SS');

   PROCEDURE trace_activity (context_in IN VARCHAR2, data_in IN CLOB);

   PROCEDURE trace_activity (context_in IN VARCHAR2, data_in IN BOOLEAN);

   /* Use this inside expressions and SQL. Always returns 1.*/
   FUNCTION trace_activity (context_in IN VARCHAR2, data_in IN VARCHAR2)
      RETURN PLS_INTEGER;

   FUNCTION trace_activity_force (context_in IN VARCHAR2, data_in IN VARCHAR2)
      RETURN PLS_INTEGER;

   PROCEDURE trace_activity_force (context_in IN VARCHAR2, data_in IN VARCHAR2);

   PROCEDURE trace_activity_force (context_in IN VARCHAR2, data_in IN CLOB);

   PROCEDURE trace_activity_force (context_in IN VARCHAR2, data_in IN BOOLEAN);

   /* Make it really easy to add force to a script*/

   PROCEDURE taf (data_in IN VARCHAR2);

   PROCEDURE taf (data_in IN CLOB);

   PROCEDURE taf (data_in IN BOOLEAN);

   PROCEDURE clear_trace_entries (before_in IN DATE DEFAULT NULL);

   PROCEDURE write_trace_to_file (dir_in IN VARCHAR2, file_in IN VARCHAR2);

   PROCEDURE trace_activity_if (condition_in   IN BOOLEAN,
                                context_in     IN VARCHAR2,
                                data_in        IN VARCHAR2);

   PROCEDURE trace_activity_if (condition_in   IN BOOLEAN,
                                context_in     IN BOOLEAN,
                                data_in        IN VARCHAR2);

   FUNCTION trace_activity_if_sql (equals_1_in   IN PLS_INTEGER,
                                   context_in    IN VARCHAR2,
                                   data_in       IN VARCHAR2)
      RETURN PLS_INTEGER;

   /* End Trace Functionality */

   /* Substitutes for DBMS_OUTPUT.PUT_LINE */

   PROCEDURE pl (string_in          IN VARCHAR2,
                 show_null_in       IN BOOLEAN DEFAULT TRUE,
                 to_log_in          IN BOOLEAN DEFAULT FALSE,
                 log_context_in     IN VARCHAR2 DEFAULT NULL,
                 to_collection_in   IN BOOLEAN DEFAULT FALSE,
                 indent_in          IN INTEGER DEFAULT 0,
                 prefix_in          IN VARCHAR2 DEFAULT '|');

   PROCEDURE putline (data_in IN VARCHAR2);

   PROCEDURE putline (data_in IN BOOLEAN);

   PROCEDURE putline (data_in     IN DATE,
                      format_in   IN VARCHAR2 DEFAULT 'YYYY-MM-DD HH24:MI:SS');

   PROCEDURE putline (clob_in            IN CLOB,
                      append_in          IN BOOLEAN DEFAULT TRUE,
                      preserve_code_in   IN BOOLEAN DEFAULT FALSE);

   /* END Substitute for DBMS_OUTPUT.PUT_LINE */

END;
/