CREATE TYPE food_t AS OBJECT
	(name VARCHAR2 (100)
  , food_group VARCHAR2 (100)
  , grown_in VARCHAR2 (100)
  , MAP MEMBER FUNCTION food_mapping
		 RETURN NUMBER
	)
	NOT FINAL;
/

CREATE OR REPLACE TYPE BODY food_t
IS
	MAP MEMBER FUNCTION food_mapping
		RETURN NUMBER
	IS
	BEGIN
		RETURN (CASE self.food_group
					  WHEN 'PROTEIN' THEN 30000
					  WHEN 'LIQUID' THEN 20000
					  WHEN 'CARBOHYDRATE' THEN 15000
					  WHEN 'VEGETABLE' THEN 10000
				  END
				  + LENGTH (self.name));
	END;
END;
/

CREATE TABLE meals
(
	served_on	  DATE
 , main_course   food_t
)
/

BEGIN
	-- Populate the meal table
	INSERT INTO meals
		 VALUES (
						SYSDATE
					 , food_t ('Shrimp cocktail', 'PROTEIN', 'Ocean')
				  );

	INSERT INTO meals
		 VALUES (
						SYSDATE + 1
					 , food_t ('Stir fry tofu', 'PROTEIN', 'Wok')
				  );

	INSERT INTO meals
		 VALUES (
						SYSDATE + 1
					 , food_t ('Peanut Butter Sandwich', 'CARBOHYDRATE', 'Kitchen')
				  );

	INSERT INTO meals
		 VALUES (SYSDATE + 1
				 , food_t ('Brussels Sprouts', 'VEGETABLE', 'Backyard')
				  );

	COMMIT;
END;
/

  SELECT TREAT (main_course AS food_t).name
	 FROM meals
ORDER BY main_course
/

DECLARE
   ot1   food_t := food_t ('Eggs benedict', 'PROTEIN', 'Farm');
   ot2   food_t := food_t ('Brussels Sprouts', 'VEGETABLE', 'Backyard');
   ot3   food_t := food_t ('Brussels Sprouts', 'VEGETABLE', 'Backyard');
BEGIN
   IF ot1 = ot2
   THEN
      DBMS_OUTPUT.put_line ('equal - incorrect');
   ELSE
      DBMS_OUTPUT.put_line ('not equal - correct');
   END IF;

   IF ot2 <> ot3
   THEN
      DBMS_OUTPUT.put_line ('not equal - incorrect');
   ELSE
      DBMS_OUTPUT.put_line ('equal - correct');
   END IF;
END;
/
