/* 
Determine type of function from ALL_IDENTIFIERS

Provide by Joop Leendertse - J.Leendertse@cb.nl 

*/
WITH parameters
     AS (  SELECT p.object_name,
                  p.object_type,
                  p.name parameter_name,
                  TRIM (REPLACE (p.TYPE, 'FORMAL')) parameter_type,
                  p.usage_context_id,
                  t.name || CASE WHEN c.name IS NOT NULL THEN '.' || c.name END
                     data_type,
                  p.usage_id sequence
             FROM user_identifiers p
                  JOIN user_identifiers t
                     ON     p.object_name = t.object_name
                        AND p.object_type = t.object_type
                        AND p.usage_id = t.usage_context_id
                  LEFT OUTER JOIN user_identifiers c
                     ON     c.object_name = t.object_name
                        AND c.object_type = t.object_type
                        AND t.usage_id = c.usage_context_id
            WHERE     p.TYPE LIKE 'FORMAL %'
                  AND p.usage = 'DECLARATION'
                  AND t.usage = 'REFERENCE'
                  AND NVL (c.TYPE, 'COLUMN') = 'COLUMN'
         ORDER BY p.usage_id),
     package_members
     AS (SELECT p.object_name,
                p.object_type,
                p.name package_name,
                m.name member_name,
                m.TYPE member_type,
                m.usage_id
           FROM user_identifiers p, user_identifiers m
          WHERE     p.object_name = m.object_name
                AND p.object_type = m.object_type
                AND p.usage_id = m.usage_context_id
                AND p.usage = 'DECLARATION'
                AND p.TYPE = 'PACKAGE'
                AND m.TYPE IN ('PROCEDURE', 'FUNCTION')),
     function_return_type
     AS (SELECT f.object_name,
                f.object_type,
                f.name,
                r.name || CASE WHEN c.name IS NOT NULL THEN '.' || c.name END
                   data_type,
                f.usage_id
           FROM user_identifiers f
                JOIN user_identifiers r
                   ON     f.object_name = r.object_name
                      AND f.object_type = r.object_type
                      AND f.usage_id = r.usage_context_id
                LEFT OUTER JOIN user_identifiers c
                   ON     c.object_name = r.object_name
                      AND c.object_type = r.object_type
                      AND r.usage_id = c.usage_context_id
          WHERE     f.TYPE = 'FUNCTION'
                AND r.usage = 'REFERENCE'
                AND (   r.TYPE LIKE ('%DATATYPE')
                     OR r.TYPE IN ('OBJECT',
                                   'RECORD',
                                   'TABLE',
                                   'VIEW'))
                AND NVL (c.TYPE, 'COLUMN') = 'COLUMN'),
     package_info
     AS (SELECT pack.object_name,
                pack.member_name,
                pack.usage_id,
                pack.member_type,
                ret.data_type ret_data_type,
                par.parameter_name,
                par.parameter_type,
                par.data_type par_data_type,
                par.sequence
           FROM package_members pack
                LEFT OUTER JOIN parameters par
                   ON     pack.object_name = par.object_name
                      AND pack.object_type = par.object_type
                      AND pack.usage_id = par.usage_context_id
                LEFT OUTER JOIN function_return_type ret
                   ON     pack.object_name = ret.object_name
                      AND pack.object_type = ret.object_type
                      AND pack.usage_id = ret.usage_id),
     module_info
     AS (SELECT MOD.object_name,
                MOD.name,
                MOD.usage_id,
                MOD.object_type,
                ret.data_type ret_data_type,
                par.parameter_name,
                par.parameter_type,
                par.data_type par_data_type,
                par.sequence
           FROM user_identifiers MOD
                LEFT OUTER JOIN function_return_type ret
                   ON     ret.object_name = MOD.object_name
                      AND ret.object_type = MOD.object_type
                LEFT OUTER JOIN parameters par
                   ON     par.object_name = MOD.object_name
                      AND par.object_type = MOD.object_type
          WHERE     MOD.usage = 'DEFINITION'
                AND MOD.object_type IN ('FUNCTION', 'PROCEDURE'))
  SELECT *
    FROM (SELECT m.object_name,
                 m.name,
                 m.usage_id,
                 m.object_type,
                 m.ret_data_type,
                 m.parameter_name,
                 m.parameter_type,
                 m.par_data_type,
                 m.sequence
            FROM module_info m
          UNION
          SELECT p.object_name,
                 p.member_name,
                 p.usage_id,
                 p.member_type,
                 p.ret_data_type,
                 p.parameter_name,
                 p.parameter_type,
                 p.par_data_type,
                 p.sequence
            FROM package_info p)
   WHERE object_name = 'QDB_PLAYER_MGR'
ORDER BY usage_id, sequence
;