CREATE TABLE qz_tab1 (d DATE)
/

CREATE TABLE qz_tab2 (n NUMBER)
/

BEGIN
   INSERT INTO qz_tab1
        VALUES (DATE '2011-10-30');

   INSERT INTO qz_tab1
        VALUES (DATE '2014-05-20');

   INSERT INTO qz_tab2
        VALUES (100);

   INSERT INTO qz_tab2
        VALUES (200);

   COMMIT;
END;
/

CREATE TYPE qz_tab1_ot AUTHID DEFINER IS OBJECT 
(
   d date
)
/

CREATE TYPE qz_tab1_nt AS TABLE OF qz_tab1_ot;
/

CREATE TYPE qz_tab2_ot AUTHID DEFINER IS OBJECT 
(
   n number
)
/

CREATE TYPE qz_tab2_nt AS TABLE OF qz_tab2_ot;
/

CREATE OR REPLACE PACKAGE qz_pkg
   AUTHID DEFINER
IS
type tab1_nt is table of qz_tab1%ROWTYPE;
   TYPE tab1_rc IS REF CURSOR
      RETURN qz_tab1%ROWTYPE;
type tab2_nt is table of qz_tab2%ROWTYPE;

   TYPE tab2_rc IS REF CURSOR
      RETURN qz_tab2%ROWTYPE;
END;
/

CREATE OR REPLACE FUNCTION qz_streaming_tf (data1_in   IN qz_pkg.tab1_rc,
                                            data2_in   IN qz_pkg.tab2_rc,
                                            data3_in   IN qz_pkg.tab2_rc)
   RETURN qz_tab2_nt
   AUTHID DEFINER
IS
   l_data1    qz_pkg.tab1_nt;
   l_data2    qz_pkg.tab2_nt;
   l_data3    qz_pkg.tab2_nt;
   l_return   qz_tab2_nt := qz_tab2_nt();
BEGIN
   FETCH data1_in BULK COLLECT INTO l_data1;

   FETCH data2_in BULK COLLECT INTO l_data2;

   FETCH data3_in BULK COLLECT INTO l_data3;

   FOR indx IN 1 .. l_data1.COUNT
   LOOP
      l_return.EXTEND;
      l_return (l_return.LAST) := qz_tab2_ot (TO_CHAR (l_data1 (indx).d, 'YYYY'));
   END LOOP;

   FOR indx IN 1 .. l_data2.COUNT
   LOOP
      l_return.EXTEND;
      l_return (l_return.LAST) := qz_tab2_ot (l_data2 (indx).n);
   END LOOP;

   FOR indx IN 1 .. l_data3.COUNT
   LOOP
      l_return.EXTEND;
      l_return (l_return.LAST) := qz_tab2_ot (l_data3 (indx).n);
   END LOOP;

   RETURN l_return;
END;
/

SELECT *
  FROM TABLE (qz_streaming_tf (CURSOR (SELECT * FROM qz_tab1),
                               CURSOR (SELECT * FROM qz_tab2),
                               CURSOR (SELECT *
                                         FROM qz_tab2
                                        WHERE n > 100)))
/

/* Multiple TF calls in single SELECT */

CREATE OR REPLACE FUNCTION qz_streaming_tf1 (data1_in IN qz_pkg.tab1_rc)
   RETURN qz_tab2_nt
   AUTHID DEFINER
IS
   l_data1    qz_pkg.tab1_nt;
   l_return   qz_tab2_nt := qz_tab2_nt();
BEGIN
   FETCH data1_in BULK COLLECT INTO l_data1;

   FOR indx IN 1 .. l_data1.COUNT
   LOOP
      l_return.EXTEND;
      l_return (l_return.LAST) := qz_tab2_ot (TO_CHAR (l_data1 (indx).d, 'YYYY'));
   END LOOP;

   RETURN l_return;
END;
/

CREATE OR REPLACE FUNCTION qz_streaming_tf2 (data2_in IN qz_pkg.tab2_rc)
   RETURN qz_tab2_nt
   AUTHID DEFINER
IS
   l_data2    qz_pkg.tab2_nt;
   l_return   qz_tab2_nt := qz_tab2_nt();
BEGIN
   FETCH data2_in BULK COLLECT INTO l_data2;

   FOR indx IN 1 .. l_data2.COUNT
   LOOP
      l_return.EXTEND;
      l_return (l_return.LAST) := qz_tab2_ot (l_data2 (indx).n);
   END LOOP;

   RETURN l_return;
END;
/

SELECT *
  FROM TABLE (qz_streaming_tf1 (CURSOR (SELECT * FROM qz_tab1))) tf1,
       TABLE (qz_streaming_tf2 (CURSOR (SELECT * FROM qz_tab2))) tf2
/