DROP TYPE pet_ot FORCE;

DROP TYPE pet_nt FORCE;

CREATE TYPE pet_ot IS OBJECT
(
   name VARCHAR2 (10),
   breed VARCHAR2 (20),
   date_of_birth DATE
);
/

CREATE TYPE pet_nt IS TABLE OF pet_ot;
/

CREATE OR REPLACE FUNCTION pet_family (dad_in IN pet_ot, mom_in IN pet_ot)
   RETURN pet_nt
   AUTHID DEFINER
IS
   l_family   pet_nt := pet_nt (dad_in, mom_in);
BEGIN
   FOR indx IN 1 ..
               CASE mom_in.breed
                  WHEN 'RABBIT' THEN 12
                  WHEN 'DOG' THEN 4
                  WHEN 'KANGAROO' THEN 1
               END
   LOOP
      l_family.EXTEND;
      l_family (l_family.LAST) :=
         pet_ot ('BABY' || indx,
                 mom_in.breed,
                 ADD_MONTHS (SYSDATE, -1 * DBMS_RANDOM.VALUE (1, 6)));
   END LOOP;

   RETURN l_family;
END;
/

SELECT *
  FROM TABLE (
          pet_family (pet_ot ('Hoppy', 'RABBIT', SYSDATE - 500),
                      pet_ot ('Hippy', 'RABBIT', SYSDATE - 300)))
/                      

REM Pipelined version

CREATE OR REPLACE FUNCTION pet_family_pl (dad_in IN pet_ot, mom_in IN pet_ot)
   RETURN pet_nt
   AUTHID DEFINER
   PIPELINED
IS
BEGIN
   PIPE ROW (dad_in);
   PIPE ROW (mom_in);

   FOR indx IN 1 ..
               CASE mom_in.breed
                  WHEN 'RABBIT' THEN 12
                  WHEN 'DOG' THEN 4
                  WHEN 'KANGAROO' THEN 1
               END
   LOOP
      PIPE ROW (pet_ot ('BABY' || indx,
                        mom_in.breed,
                        ADD_MONTHS (SYSDATE, -1 * DBMS_RANDOM.VALUE (1, 6))));
   END LOOP;

   RETURN;
END;
/

SELECT *
  FROM TABLE (
          pet_family_pl (pet_ot ('Bob', 'KANGAROO', SYSDATE - 1000),
                         pet_ot ('Sally', 'KANGAROO', SYSDATE - 700)))
/                         