DROP TABLE my_trace_control
/

DROP TABLE my_trace
/

CREATE TABLE my_trace_control (trace_on VARCHAR2 (1))
/

BEGIN
   INSERT INTO my_trace_control
        VALUES ('N');
END;
/

CREATE TABLE my_trace
(
   id              NUMBER GENERATED ALWAYS AS IDENTITY,
   trace_context   VARCHAR2 (1000),
   trace_info      VARCHAR2 (4000),
   trace_stack     VARCHAR2 (4000),
   created_on      DATE,
   created_by      VARCHAR2 (128)
)
/

CREATE OR REPLACE PACKAGE trace_pkg
   AUTHID DEFINER
IS
   g_use_shortcut   BOOLEAN := TRUE;

   PROCEDURE turn_on_trace;

   PROCEDURE turn_off_trace;

   FUNCTION trace_is_on RETURN BOOLEAN;

   PROCEDURE traceit (context_in IN VARCHAR2, info_in IN VARCHAR2);
END;
/

CREATE OR REPLACE PACKAGE BODY trace_pkg
IS
   g_check_trace_secs   CONSTANT INTEGER := 20;
   g_last_trace_check_on         DATE;
   g_last_trace_check            BOOLEAN;

   PROCEDURE turn_on_trace
   IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      UPDATE my_trace_control
         SET trace_on = 'Y';

      COMMIT;

      g_last_trace_check_on := sysdate;
      g_last_trace_check := TRUE;
   END;

   PROCEDURE turn_off_trace
   IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      UPDATE my_trace_control
         SET trace_on = 'N';

      COMMIT;

      g_last_trace_check_on := NULL;
      g_last_trace_check := FALSE;
   END;

   FUNCTION trace_is_on RETURN BOOLEAN
   IS
      l_trace    my_trace_control.trace_on%TYPE;
      l_return   BOOLEAN DEFAULT FALSE;
   BEGIN
      IF     g_use_shortcut
         AND (SYSDATE - g_last_trace_check_on) * (24 * 60 * 60) <=
                g_check_trace_secs
      THEN
         l_return := g_last_trace_check;
      ELSE
         SELECT trace_on INTO l_trace FROM my_trace_control;

         l_return := l_trace = 'Y';
      END IF;

      g_last_trace_check := l_return;

      RETURN l_return;
   END trace_is_on;

   PROCEDURE traceit (context_in IN VARCHAR2, info_in IN VARCHAR2)
   IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      INSERT INTO my_trace (trace_context,
                            trace_info,
                            trace_stack,
                            created_on,
                            created_by)
           VALUES (context_in,
                   info_in,
                   DBMS_UTILITY.format_call_stack,
                   SYSDATE,
                   USER);

      COMMIT;
   END;
END;
/