/* Declare an exception in a package specifiation */

CREATE OR REPLACE PACKAGE qz_exc
IS
   e_balance_too_low EXCEPTION;
END;
/

CREATE OR REPLACE FUNCTION qz_a_string
   RETURN VARCHAR2
IS
BEGIN
   RAISE qz_exc.e_balance_too_low;
END;
/

/* Raise a user-defined exception just like a system-defined one */

DECLARE
   e_balance_too_low EXCEPTION;
BEGIN
   RAISE e_balance_too_low;
END;
/

/* Handle a user-defined exception by name in WHEN clause */

DECLARE
   e_balance_too_low EXCEPTION;
BEGIN
   RASIE e_balance_too_low;
EXCEPTION
   WHEN e_balance_too_low
   THEN
      DBMS_OUTPUT.put_line (DBMS_UTILITY.format_error_stack);
END;
/

/* Assign an error code <> 1 for a user-defined exception 

  
*/

DECLARE
   e_balance_too_low EXCEPTION;
   PRAGMA EXCEPTION_INIT (e_balance_too_low, -20200);
BEGIN
   RAISE e_balance_too_low;
EXCEPTION
   WHEN e_balance_too_low
   THEN
      DBMS_OUTPUT.put_line (DBMS_UTILITY.format_error_stack);
END;
/

/* Provide a custom error message for a user-defined exception */

DECLARE
   e_balance_too_low EXCEPTION;
   PRAGMA EXCEPTION_INIT (e_balance_too_low, -20200);
BEGIN
   RAISE_APPLICATION_ERROR (-20200,  'Your balance has dropped below the minimum.');
EXCEPTION
   WHEN e_balance_too_low
   THEN
      DBMS_OUTPUT.put_line (DBMS_UTILITY.format_error_stack);
END;
/
