BEGIN
   alert_mgr.start_check_alerts_job (every_n_minutes_in => 1);
--   alert_mgr.change_check_frequency(1);
END;
/

BEGIN
   /* Set up email using APEX and the default API from the utility */
   alert_mgr.register_email_proc (
      proc_name_in        => 'alert_email_mgr.send_email',
      workspace_name_in   => 'MIKE');
END;
/

BEGIN
   alert_mgr.register_user ('steven.feuerstein@oracle.com');
END;
/

DECLARE
   l_actions   alert_mgr.actions_t;
BEGIN
   alert_mgr.register_alert (
      alert_name_in             => 'Always True',
      description_in            => 'Trigger is TRUE, display string',
      trigger_code_in           => 'TRUE',
      trigger_type_in           => 'RETURNTRUE',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'D',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''PROC EXECUTED'')',
      send_email_to_in          => '',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Send Email',
      description_in            => 'Trigger is TRUE, send email',
      trigger_code_in           => 'TRUE',
      trigger_type_in           => 'RETURNTRUE',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'D',
      ignore_failures_in        => 'Y',
      action_type_in            => 'EMAIL',
      action_code_in            => NULL,
      send_email_to_in          => 'steven.feuerstein@oracle.com',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Check one row',
      description_in            => 'Always returns one row, no more than once an hour',
      trigger_code_in           => 'select count(*) from dual',
      trigger_type_in           => 'ONEROW',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'H',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''One row found'')',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Check no rows - valid',
      description_in            => 'Always returns no rows, no more than every 3 hours',
      trigger_code_in           => 'select count(*) from dual where 1 = 2',
      trigger_type_in           => 'NOROWS',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => '180',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''No rows found'')',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Check one row none returned',
      description_in            => 'Always returns no rows',
      trigger_code_in           => 'select count(*) from dual where 1 = 2',
      trigger_type_in           => 'ONEROW',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'D',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''No rows found - Should NOT see this'')',
      is_active_in              => 'Y');

   /* Multiple actions for one alert  */
   l_actions (1).action_type := 'PROC';
   l_actions (1).action_code := 'alert_mgr.log(''Action 1'')';
   l_actions (2).action_type := 'EMAIL';
   l_actions (2).send_email_to := 'steven.feuerstein@oracle.com';
   l_actions (2).email_subject := 'Subject is action 2';
   l_actions (2).email_body := '<p>The body of the email</p>';

   alert_mgr.register_alert (alert_name_in             => 'Two actions',
                             description_in            => 'Use the array approach',
                             trigger_code_in           => 'select count(*) from dual',
                             trigger_type_in           => 'ONEROW',
                             check_frequency_in        => 'H',
                             recurrence_frequency_in   => 'H',
                             ignore_failures_in        => 'Y',
                             actions_in                => l_actions,
                             is_active_in              => 'Y');
END;
/

BEGIN
   alert_mgr.register_user ('steven.feuerstein@oracle.com');
   alert_mgr.follow_alert ('Two actions', 'steven.feuerstein@oracle.com');

   FOR rec IN (SELECT u.email_address, a.alert_name
                 FROM alert_follows f, alert_users u, alerts a
                WHERE f.user_id = u.id AND f.alert_id = a.id)
   LOOP
      DBMS_OUTPUT.put_line (rec.email_address || ' following ' || rec.alert_name);
   END LOOP;
END;
/

/* Use stop_checking to disable alert checking even if and when the job wakes up */

BEGIN
   --alert_mgr.stop_checking;
   alert_mgr.resume_checking;
END;
/

/* Check job details */

SELECT *
  FROM user_scheduler_jobs
 WHERE job_name = 'CHECK_ALERTS_JOB'
/

BEGIN
   alert_mgr.bpl (alert_mgr.checking_for_alerts);
END;
/

ALTER SESSION SET nls_date_format='YYYY-MM-DD HH24:MI:SS'
/

SELECT * FROM alert_config
/

  SELECT *
    FROM alerts
ORDER BY created DESC
/

  SELECT *
    FROM alert_log
ORDER BY created DESC
/

  SELECT *
    FROM alert_runs
ORDER BY created DESC
/

  SELECT to_char (sysdate, 'YYYY-MM-DD HH24:MI:SS'), to_char (completed_at, 'YYYY-MM-DD HH24:MI:SS') completed
    FROM alert_runs
ORDER BY created DESC
/

  SELECT *
    FROM apex_mail_log
ORDER BY last_updated_on DESC;
/

  SELECT *
    FROM apex_mail_queue
ORDER BY last_updated_on DESC;
/

DELETE FROM alert_runs;

DELETE FROM alert_log;