CREATE TABLE earthquake_alerts
(
   id              NUMBER NOT NULL CONSTRAINT earthquake_alert_id_pk PRIMARY KEY,
   earthquake_id   VARCHAR2 (100)
                      CONSTRAINT earthquake_alert_eq_id_fk
                          REFERENCES earthquakes (id) ON DELETE CASCADE,
   notified_at     DATE,
   created         DATE NOT NULL,
   created_by      VARCHAR2 (255) NOT NULL,
   updated         DATE NOT NULL,
   updated_by      VARCHAR2 (255) NOT NULL
)
/

CREATE OR REPLACE TRIGGER earthquake_alerts_biu
   BEFORE INSERT OR UPDATE
   ON earthquake_alerts
   FOR EACH ROW
BEGIN
   IF :new.id IS NULL
   THEN
      :new.id := TO_NUMBER (SYS_GUID (), 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX');
   END IF;

   IF INSERTING
   THEN
      :new.created := SYSDATE;
      :new.created_by := NVL (SYS_CONTEXT ('APEX$SESSION', 'APP_USER'), USER);
   END IF;

   :new.updated := SYSDATE;
   :new.updated_by := NVL (SYS_CONTEXT ('APEX$SESSION', 'APP_USER'), USER);
END;
/

CREATE OR REPLACE PACKAGE earthquake_mgr
   AUTHID DEFINER
IS
   PROCEDURE record_notifications;
END;
/

CREATE OR REPLACE PACKAGE BODY earthquake_mgr
IS
   PROCEDURE record_notifications
   IS
   BEGIN
      INSERT INTO earthquake_alerts (earthquake_id, notified_at)
         SELECT id, SYSDATE
           FROM earthquakes
          WHERE     magnitude >= 6.5
                AND id NOT IN (SELECT earthquake_id FROM earthquake_alerts);
      alert_mgr.log ('Notified on ' || SQL%ROWCOUNT || ' earthquakes > 6.5');
      COMMIT;
   END;
END;
/

